@@ -44,7 +44,8 @@
 #![allow(
     clippy::match_single_binding,
     clippy::inconsistent_struct_constructor,
-    clippy::assign_op_pattern
+    clippy::assign_op_pattern,
+    clippy::len_without_is_empty
 )]
 
 #[cfg(test)]
@@ -74,6 +75,7 @@ mod publish;
 mod remove;
 pub mod run;
 mod shell;
+mod text_layout;
 
 use config::root_config;
 use fs::{get_current_directory, get_project_root};
@@ -111,6 +113,7 @@ struct TreeOptions {
         short,
         long,
         ignore_case = true,
+        conflicts_with = "invert",
         help = "Package to be used as the root of the tree"
     )]
     package: Option<String>,
@@ -119,6 +122,7 @@ struct TreeOptions {
         short,
         long,
         ignore_case = true,
+        conflicts_with = "package",
         help = "Invert the tree direction and focus on the given package",
         value_name = "PACKAGE"
     )]
@@ -215,6 +219,8 @@ enum Command {
     Shell,
 
     /// Run the project
+    ///
+    /// This command runs the `main` function from the `<PROJECT_NAME>` module.
     #[command(trailing_var_arg = true)]
     Run {
         #[arg(short, long, ignore_case = true, help = target_doc())]
@@ -235,6 +241,8 @@ enum Command {
     },
 
     /// Run the project tests
+    ///
+    /// This command runs the `main` function from the `<PROJECT_NAME>_test` module.
     #[command(trailing_var_arg = true)]
     Test {
         #[arg(short, long, ignore_case = true, help = target_doc())]
@@ -247,6 +255,8 @@ enum Command {
     },
 
     /// Run the project development entrypoint
+    ///
+    /// This command runs the `main` function from the `<PROJECT_NAME>_dev` module.
     #[command(trailing_var_arg = true)]
     Dev {
         #[arg(short, long, ignore_case = true, help = target_doc())]
@@ -395,6 +405,9 @@ enum Dependencies {
     /// Download all dependency packages
     Download,
 
+    /// List all outdated dependencies
+    Outdated,
+
     /// Update dependency packages to their latest versions
     Update(UpdateOptions),
 
@@ -579,6 +592,11 @@ fn parse_and_run_command() -> Result<(), Error> {
             download_dependencies(&paths)
         }
 
+        Command::Deps(Dependencies::Outdated) => {
+            let paths = find_project_paths()?;
+            dependencies::outdated(&paths)
+        }
+
         Command::Deps(Dependencies::Update(options)) => {
             let paths = find_project_paths()?;
             dependencies::update(&paths, options.packages)
