@@ -142,13 +142,17 @@ fn check_for_name_squatting(package: &Package) -> Result<(), Error> {
         return Ok(());
     }
 
-    let definitions = &module.ast.definitions;
-
-    if definitions.len() > 2 {
+    if module.ast.definitions_len() > 2 {
         return Ok(());
     }
 
-    let Some(main) = definitions.iter().find_map(|d| d.main_function()) else {
+    let Some(main) = module
+        .ast
+        .definitions
+        .functions
+        .iter()
+        .find_map(|function| function.main_function())
+    else {
         return Ok(());
     };
 
@@ -169,8 +173,8 @@ fn check_for_default_main(package: &Package) -> Result<(), Error> {
     let has_default_main = package
         .modules
         .iter()
-        .flat_map(|m| m.ast.definitions.iter())
-        .filter_map(|d| d.main_function())
+        .flat_map(|module| module.ast.definitions.functions.iter())
+        .filter_map(|function| function.main_function())
         .any(|main| main.documentation.is_none() && is_default_main(main, package_name));
 
     if has_default_main {
@@ -439,6 +443,31 @@ fn do_build_hex_tarball(paths: &ProjectPaths, config: &mut PackageConfig) -> Res
         });
     }
 
+    // empty_modules is a list of modules that do not export any values or types.
+    // We do not allow publishing packages that contain empty modules.
+    let empty_modules: Vec<_> = built
+        .root_package
+        .modules
+        .iter()
+        .filter(|module| {
+            built
+                .module_interfaces
+                .get(&module.name)
+                .map(|interface| {
+                    // Check if the module exports any values or types
+                    interface.values.is_empty() && interface.types.is_empty()
+                })
+                .unwrap_or(false)
+        })
+        .map(|module| module.name.clone())
+        .collect();
+
+    if !empty_modules.is_empty() {
+        return Err(Error::CannotPublishEmptyModules {
+            unfinished: empty_modules,
+        });
+    }
+
     // TODO: If any of the modules in the package contain a leaked internal type then
     // refuse to publish as the package is not yet finished.
     // We need to move aliases in to the type system first.
