@@ -6,6 +6,7 @@ use super::*;
 use crate::{
     build::ExpressionPosition,
     exhaustiveness::CompiledCase,
+    parse::LiteralFloatValue,
     type_::{HasType, Type, ValueConstructorVariant, bool},
 };
 
@@ -22,6 +23,7 @@ pub enum TypedExpr {
         location: SrcSpan,
         type_: Arc<Type>,
         value: EcoString,
+        float_value: LiteralFloatValue,
     },
 
     String {
@@ -105,6 +107,15 @@ pub enum TypedExpr {
         documentation: Option<EcoString>,
     },
 
+    /// Generated internally for accessing unlabelled fields of a custom type,
+    /// such as for record updates.
+    PositionalAccess {
+        location: SrcSpan,
+        type_: Arc<Type>,
+        index: u64,
+        record: Box<Self>,
+    },
+
     ModuleSelect {
         location: SrcSpan,
         field_start: u32,
@@ -216,7 +227,8 @@ impl TypedExpr {
             | Self::Int { .. }
             | Self::Float { .. }
             | Self::String { .. }
-            | Self::Invalid { .. } => self.self_if_contains_location(byte_index),
+            | Self::Invalid { .. }
+            | Self::PositionalAccess { .. } => self.self_if_contains_location(byte_index),
 
             Self::ModuleSelect {
                 location,
@@ -422,7 +434,8 @@ impl TypedExpr {
             | Self::Float { .. }
             | Self::String { .. }
             | Self::ModuleSelect { .. }
-            | Self::Invalid { .. } => None,
+            | Self::Invalid { .. }
+            | Self::PositionalAccess { .. } => None,
 
             Self::Pipeline {
                 first_value,
@@ -584,7 +597,7 @@ impl TypedExpr {
         }
     }
 
-    pub fn non_zero_compile_time_number(&self) -> bool {
+    pub fn is_non_zero_compile_time_number(&self) -> bool {
         match self {
             Self::Int { int_value, .. } => int_value != &BigInt::ZERO,
             Self::Float { value, .. } => is_non_zero_number(value),
@@ -592,7 +605,7 @@ impl TypedExpr {
         }
     }
 
-    pub fn zero_compile_time_number(&self) -> bool {
+    pub fn is_zero_compile_time_number(&self) -> bool {
         match self {
             Self::Int { int_value, .. } => int_value == &BigInt::ZERO,
             Self::Float { value, .. } => !is_non_zero_number(value),
@@ -623,6 +636,7 @@ impl TypedExpr {
             | Self::TupleIndex { location, .. }
             | Self::ModuleSelect { location, .. }
             | Self::RecordAccess { location, .. }
+            | Self::PositionalAccess { location, .. }
             | Self::RecordUpdate { location, .. }
             | Self::Invalid { location, .. } => *location,
         }
@@ -650,6 +664,7 @@ impl TypedExpr {
             | Self::TupleIndex { location, .. }
             | Self::ModuleSelect { location, .. }
             | Self::RecordAccess { location, .. }
+            | Self::PositionalAccess { location, .. }
             | Self::RecordUpdate { location, .. }
             | Self::Invalid { location, .. } => *location,
             Self::Block { statements, .. } => statements.last().location(),
@@ -677,6 +692,7 @@ impl TypedExpr {
             | TypedExpr::BitArray { .. }
             | TypedExpr::TupleIndex { .. }
             | TypedExpr::RecordAccess { .. }
+            | TypedExpr::PositionalAccess { .. }
             | Self::Invalid { .. } => None,
 
             // TODO: test
@@ -719,6 +735,7 @@ impl TypedExpr {
             | Self::TupleIndex { type_, .. }
             | Self::ModuleSelect { type_, .. }
             | Self::RecordAccess { type_, .. }
+            | Self::PositionalAccess { type_, .. }
             | Self::RecordUpdate { type_, .. }
             | Self::Invalid { type_, .. } => type_.clone(),
             Self::Pipeline { finally, .. } => finally.type_(),
@@ -740,7 +757,7 @@ impl TypedExpr {
 
             // Calls are literals if they are records and all the arguemnts are also literals.
             Self::Call { fun, arguments, .. } => {
-                fun.is_record_builder()
+                fun.is_record_literal()
                     && arguments.iter().all(|argument| argument.value.is_literal())
             }
 
@@ -811,6 +828,7 @@ impl TypedExpr {
             | TypedExpr::RecordUpdate { .. }
             | TypedExpr::NegateBool { .. }
             | TypedExpr::NegateInt { .. }
+            | TypedExpr::PositionalAccess { .. }
             | TypedExpr::Invalid { .. } => None,
         }
     }
@@ -848,6 +866,7 @@ impl TypedExpr {
             | TypedExpr::Var { .. }
             | TypedExpr::BinOp { .. }
             | TypedExpr::RecordAccess { .. }
+            | TypedExpr::PositionalAccess { .. }
             | TypedExpr::TupleIndex { .. }
             | TypedExpr::RecordUpdate { .. }
             | TypedExpr::Fn { .. } => true,
@@ -879,7 +898,7 @@ impl TypedExpr {
             }
 
             TypedExpr::Call { fun, arguments, .. } => {
-                (fun.is_record_builder() || fun.called_function_purity().is_pure())
+                (fun.is_record_literal() || fun.called_function_purity().is_pure())
                     && arguments
                         .iter()
                         .all(|argument| argument.value.is_pure_value_constructor())
@@ -972,6 +991,7 @@ impl TypedExpr {
             | TypedExpr::RecordUpdate { .. }
             | TypedExpr::NegateBool { .. }
             | TypedExpr::NegateInt { .. }
+            | TypedExpr::PositionalAccess { .. }
             | TypedExpr::Invalid { .. } => Purity::Unknown,
         }
     }
@@ -980,9 +1000,9 @@ impl TypedExpr {
     /// Returns true if the value is a literal record builder like
     /// `Wibble(1, 2)`, `module.Wobble("a")`
     ///
-    pub fn is_record_builder(&self) -> bool {
+    pub fn is_record_literal(&self) -> bool {
         match self {
-            TypedExpr::Call { fun, .. } => fun.is_record_builder(),
+            TypedExpr::Call { fun, .. } => fun.is_record_literal(),
             TypedExpr::Var { constructor, .. } => constructor.variant.is_record(),
             TypedExpr::ModuleSelect {
                 constructor: ModuleValueConstructor::Record { .. },
@@ -992,6 +1012,29 @@ impl TypedExpr {
         }
     }
 
+    /// Returns true if the expression is a record constructor function/record constructor
+    /// with non-zero arity.
+    ///
+    pub fn is_record_constructor_function(&self) -> bool {
+        match self {
+            TypedExpr::Var {
+                constructor:
+                    ValueConstructor {
+                        variant: ValueConstructorVariant::Record { arity, .. },
+                        ..
+                    },
+                ..
+            } => *arity > 0,
+
+            TypedExpr::ModuleSelect {
+                constructor: ModuleValueConstructor::Record { arity, .. },
+                ..
+            } => *arity > 0,
+
+            _ => false,
+        }
+    }
+
     /// If the given expression is a literal record builder, this will return
     /// index of the variant being built.
     ///
@@ -1077,6 +1120,7 @@ impl TypedExpr {
             | TypedExpr::BinOp { location, .. }
             | TypedExpr::Case { location, .. }
             | TypedExpr::RecordAccess { location, .. }
+            | TypedExpr::PositionalAccess { location, .. }
             | TypedExpr::ModuleSelect { location, .. }
             | TypedExpr::Tuple { location, .. }
             | TypedExpr::TupleIndex { location, .. }
@@ -1108,6 +1152,7 @@ impl TypedExpr {
             | TypedExpr::BinOp { .. }
             | TypedExpr::Case { .. }
             | TypedExpr::RecordAccess { .. }
+            | TypedExpr::PositionalAccess { .. }
             | TypedExpr::Tuple { .. }
             | TypedExpr::TupleIndex { .. }
             | TypedExpr::Todo { .. }
@@ -1134,6 +1179,322 @@ impl TypedExpr {
             _ => false,
         }
     }
+
+    /// Checks that two expressions are written in the same (ignoring
+    /// whitespace).
+    ///
+    /// This is useful for the language server to know when it is possible to
+    /// merge two blocks of code together because they are the same.
+    /// Simply checking for equality of the AST nodes wouldn't work as those
+    /// also contain the source location (meaning that two expression that look
+    /// the same but are in different places would be considered different)!
+    ///
+    pub(crate) fn syntactically_eq(&self, other: &TypedExpr) -> bool {
+        match (self, other) {
+            (TypedExpr::Int { int_value: n, .. }, TypedExpr::Int { int_value: m, .. }) => n == m,
+            (TypedExpr::Int { .. }, _) => false,
+
+            (TypedExpr::Float { float_value: n, .. }, TypedExpr::Float { float_value: m, .. }) => {
+                n == m
+            }
+            (TypedExpr::Float { .. }, _) => false,
+
+            (TypedExpr::String { value, .. }, TypedExpr::String { value: other, .. }) => {
+                value == other
+            }
+            (TypedExpr::String { .. }, _) => false,
+
+            (
+                TypedExpr::Block { statements, .. },
+                TypedExpr::Block {
+                    statements: other, ..
+                },
+            ) => pairwise_all(statements, other, |(one, other)| {
+                one.syntactically_eq(other)
+            }),
+
+            (TypedExpr::Block { .. }, _) => false,
+
+            (
+                TypedExpr::List { elements, tail, .. },
+                TypedExpr::List {
+                    elements: other_elements,
+                    tail: other_tail,
+                    ..
+                },
+            ) => {
+                let tails_are_equal = match (tail, other_tail) {
+                    (Some(one), Some(other)) => one.syntactically_eq(other),
+                    (None, Some(_)) | (Some(_), None) => false,
+                    (None, None) => true,
+                };
+
+                tails_are_equal
+                    && pairwise_all(elements, other_elements, |(one, other)| {
+                        one.syntactically_eq(other)
+                    })
+            }
+            (TypedExpr::List { .. }, _) => false,
+
+            (TypedExpr::Var { name, .. }, TypedExpr::Var { name: other, .. }) => name == other,
+            (TypedExpr::Var { .. }, _) => false,
+
+            (
+                TypedExpr::Fn {
+                    arguments, body, ..
+                },
+                TypedExpr::Fn {
+                    arguments: other_arguments,
+                    body: other_body,
+                    ..
+                },
+            ) => {
+                let arguments_are_equal =
+                    pairwise_all(arguments, other_arguments, |(one, other)| {
+                        one.get_variable_name() == other.get_variable_name()
+                    });
+
+                let bodies_are_equal =
+                    pairwise_all(body, other_body, |(one, other)| one.syntactically_eq(other));
+
+                arguments_are_equal && bodies_are_equal
+            }
+            (TypedExpr::Fn { .. }, _) => false,
+
+            (
+                TypedExpr::Pipeline {
+                    first_value,
+                    assignments,
+                    finally,
+                    ..
+                },
+                TypedExpr::Pipeline {
+                    first_value: other_first_value,
+                    assignments: other_assignments,
+                    finally: other_finally,
+                    ..
+                },
+            ) => {
+                first_value.value.syntactically_eq(&other_first_value.value)
+                    && pairwise_all(assignments, other_assignments, |(one, other)| {
+                        one.0.value.syntactically_eq(&other.0.value)
+                    })
+                    && finally.syntactically_eq(other_finally)
+            }
+            (TypedExpr::Pipeline { .. }, _) => false,
+
+            (
+                TypedExpr::Call { fun, arguments, .. },
+                TypedExpr::Call {
+                    fun: other_fun,
+                    arguments: other_arguments,
+                    ..
+                },
+            ) => {
+                fun.syntactically_eq(other_fun)
+                    && pairwise_all(arguments, other_arguments, |(one, other)| {
+                        one.label == other.label && one.value.syntactically_eq(&other.value)
+                    })
+            }
+            (TypedExpr::Call { .. }, _) => false,
+
+            (
+                TypedExpr::BinOp {
+                    name, left, right, ..
+                },
+                TypedExpr::BinOp {
+                    name: other_name,
+                    left: other_left,
+                    right: other_right,
+                    ..
+                },
+            ) => {
+                name == other_name
+                    && left.syntactically_eq(other_left)
+                    && right.syntactically_eq(other_right)
+            }
+            (TypedExpr::BinOp { .. }, _) => false,
+
+            (
+                TypedExpr::Case {
+                    subjects, clauses, ..
+                },
+                TypedExpr::Case {
+                    subjects: other_subjects,
+                    clauses: other_clauses,
+                    ..
+                },
+            ) => {
+                pairwise_all(subjects, other_subjects, |(one, other)| {
+                    one.syntactically_eq(other)
+                }) && pairwise_all(clauses, other_clauses, |(one, other)| {
+                    one.syntactically_eq(other)
+                })
+            }
+            (TypedExpr::Case { .. }, _) => false,
+
+            (
+                TypedExpr::RecordAccess { label, record, .. },
+                TypedExpr::RecordAccess {
+                    label: other_label,
+                    record: other_record,
+                    ..
+                },
+            ) => label == other_label && record.syntactically_eq(other_record),
+            (TypedExpr::RecordAccess { .. }, _) => false,
+
+            (
+                TypedExpr::ModuleSelect {
+                    label,
+                    module_alias,
+                    ..
+                },
+                TypedExpr::ModuleSelect {
+                    label: other_label,
+                    module_alias: other_module_alias,
+                    ..
+                },
+            ) => label == other_label && module_alias == other_module_alias,
+            (TypedExpr::ModuleSelect { .. }, _) => false,
+
+            (
+                TypedExpr::Tuple { elements, .. },
+                TypedExpr::Tuple {
+                    elements: other_elements,
+                    ..
+                },
+            ) => pairwise_all(elements, other_elements, |(one, other)| {
+                one.syntactically_eq(other)
+            }),
+            (TypedExpr::Tuple { .. }, _) => false,
+
+            (
+                TypedExpr::TupleIndex { index, tuple, .. },
+                TypedExpr::TupleIndex {
+                    index: other_index,
+                    tuple: other_tuple,
+                    ..
+                },
+            ) => index == other_index && tuple.syntactically_eq(other_tuple),
+            (TypedExpr::TupleIndex { .. }, _) => false,
+
+            (
+                TypedExpr::Todo { message, kind, .. },
+                TypedExpr::Todo {
+                    message: other_message,
+                    kind: other_kind,
+                    ..
+                },
+            ) => {
+                let messages_are_equal = match (message, other_message) {
+                    (Some(one), Some(other)) => one.syntactically_eq(other),
+                    (None, None) => true,
+                    (None, Some(_)) | (Some(_), None) => false,
+                };
+                messages_are_equal && kind == other_kind
+            }
+            (TypedExpr::Todo { .. }, _) => false,
+
+            (
+                TypedExpr::Panic { message, .. },
+                TypedExpr::Panic {
+                    message: message_other,
+                    ..
+                },
+            ) => match (message, message_other) {
+                (None, None) => true,
+                (None, Some(_)) | (Some(_), None) => false,
+                (Some(one), Some(other)) => one.syntactically_eq(other),
+            },
+            (TypedExpr::Panic { .. }, _) => false,
+
+            (
+                TypedExpr::Echo {
+                    expression,
+                    message,
+                    ..
+                },
+                TypedExpr::Echo {
+                    expression: other_expression,
+                    message: other_message,
+                    ..
+                },
+            ) => {
+                let messages_are_equal = match (message, other_message) {
+                    (None, None) => true,
+                    (None, Some(_)) | (Some(_), None) => false,
+                    (Some(one), Some(other)) => one.syntactically_eq(other),
+                };
+                let expressions_are_equal = match (expression, other_expression) {
+                    (None, None) => true,
+                    (None, Some(_)) | (Some(_), None) => false,
+                    (Some(one), Some(other)) => one.syntactically_eq(other),
+                };
+                messages_are_equal && expressions_are_equal
+            }
+            (TypedExpr::Echo { .. }, _) => false,
+
+            (
+                TypedExpr::BitArray { segments, .. },
+                TypedExpr::BitArray {
+                    segments: other_segments,
+                    ..
+                },
+            ) => pairwise_all(segments, other_segments, |(one, other)| {
+                one.syntactically_eq(other)
+            }),
+            (TypedExpr::BitArray { .. }, _) => false,
+
+            (
+                TypedExpr::RecordUpdate {
+                    constructor,
+                    arguments,
+                    ..
+                },
+                TypedExpr::RecordUpdate {
+                    constructor: other_constructor,
+                    arguments: other_arguments,
+                    ..
+                },
+            ) => {
+                constructor.syntactically_eq(other_constructor)
+                    && pairwise_all(arguments, other_arguments, |(one, other)| {
+                        one.label == other.label && one.value.syntactically_eq(&other.value)
+                    })
+            }
+            (TypedExpr::RecordUpdate { .. }, _) => false,
+
+            (
+                TypedExpr::NegateBool { value, .. },
+                TypedExpr::NegateBool {
+                    value: other_value, ..
+                },
+            ) => value.syntactically_eq(other_value),
+            (TypedExpr::NegateBool { .. }, _) => false,
+
+            (TypedExpr::NegateInt { value: n, .. }, TypedExpr::NegateInt { value: m, .. }) => {
+                n.syntactically_eq(m)
+            }
+            (TypedExpr::NegateInt { .. }, _) => false,
+
+            (TypedExpr::PositionalAccess { .. }, _) => false,
+            (TypedExpr::Invalid { .. }, _) => false,
+        }
+    }
+
+    pub(crate) fn is_todo_with_no_message(&self) -> bool {
+        match self {
+            TypedExpr::Todo { message: None, .. } => true,
+            _ => false,
+        }
+    }
+}
+
+/// Checks that two slices have the same number of item and that the given
+/// predicate holds for all pairs of items.
+///
+pub(crate) fn pairwise_all<A>(one: &[A], other: &[A], function: impl Fn((&A, &A)) -> bool) -> bool {
+    one.len() == other.len() && one.iter().zip(other).all(function)
 }
 
 fn is_non_zero_number(value: &EcoString) -> bool {
@@ -1182,4 +1543,7 @@ pub enum InvalidExpression {
         module_name: EcoString,
         label: EcoString,
     },
+    UnknownVariable {
+        name: EcoString,
+    },
 }
