@@ -7,16 +7,17 @@ use crate::{
     analyse::Inferred,
     ast::{
         Assert, AssignName, Assignment, BinOp, BitArraySize, CallArg, Constant, Definition,
-        FunctionLiteralKind, Pattern, RecordBeingUpdated, SrcSpan, Statement, TargetedDefinition,
-        TodoKind, TypeAst, TypeAstConstructor, TypeAstFn, TypeAstHole, TypeAstTuple, TypeAstVar,
-        UntypedArg, UntypedAssert, UntypedAssignment, UntypedClause, UntypedConstant,
-        UntypedConstantBitArraySegment, UntypedCustomType, UntypedDefinition, UntypedExpr,
-        UntypedExprBitArraySegment, UntypedFunction, UntypedImport, UntypedModule,
-        UntypedModuleConstant, UntypedPattern, UntypedPatternBitArraySegment,
-        UntypedRecordUpdateArg, UntypedStatement, UntypedTypeAlias, UntypedUse,
-        UntypedUseAssignment, Use, UseAssignment,
+        FunctionLiteralKind, InvalidExpression, Pattern, RecordBeingUpdated, RecordUpdateArg,
+        SrcSpan, Statement, TailPattern, TargetedDefinition, TodoKind, TypeAst, TypeAstConstructor,
+        TypeAstFn, TypeAstHole, TypeAstTuple, TypeAstVar, UntypedArg, UntypedAssert,
+        UntypedAssignment, UntypedClause, UntypedConstant, UntypedConstantBitArraySegment,
+        UntypedCustomType, UntypedDefinition, UntypedExpr, UntypedExprBitArraySegment,
+        UntypedFunction, UntypedImport, UntypedModule, UntypedModuleConstant, UntypedPattern,
+        UntypedPatternBitArraySegment, UntypedRecordUpdateArg, UntypedStatement,
+        UntypedTailPattern, UntypedTypeAlias, UntypedUse, UntypedUseAssignment, Use, UseAssignment,
     },
     build::Target,
+    parse::LiteralFloatValue,
     type_::error::VariableOrigin,
 };
 
@@ -258,7 +259,11 @@ pub trait UntypedExprFolder: TypeAstFolder + UntypedConstantFolder + PatternFold
                 value,
                 int_value,
             } => self.fold_int(location, value, int_value),
-            UntypedExpr::Float { location, value } => self.fold_float(location, value),
+            UntypedExpr::Float {
+                location,
+                value,
+                float_value,
+            } => self.fold_float(location, value, float_value),
             UntypedExpr::String { location, value } => self.fold_string(location, value),
 
             UntypedExpr::Block {
@@ -714,8 +719,17 @@ pub trait UntypedExprFolder: TypeAstFolder + UntypedConstantFolder + PatternFold
         }
     }
 
-    fn fold_float(&mut self, location: SrcSpan, value: EcoString) -> UntypedExpr {
-        UntypedExpr::Float { location, value }
+    fn fold_float(
+        &mut self,
+        location: SrcSpan,
+        value: EcoString,
+        float_value: LiteralFloatValue,
+    ) -> UntypedExpr {
+        UntypedExpr::Float {
+            location,
+            value,
+            float_value,
+        }
     }
 
     fn fold_string(&mut self, location: SrcSpan, value: EcoString) -> UntypedExpr {
@@ -888,7 +902,7 @@ pub trait UntypedExprFolder: TypeAstFolder + UntypedConstantFolder + PatternFold
         &mut self,
         location: SrcSpan,
         constructor: Box<UntypedExpr>,
-        record: RecordBeingUpdated,
+        record: RecordBeingUpdated<UntypedExpr>,
         arguments: Vec<UntypedRecordUpdateArg>,
     ) -> UntypedExpr {
         UntypedExpr::RecordUpdate {
@@ -937,11 +951,19 @@ pub trait UntypedConstantFolder {
                 int_value,
             } => self.fold_constant_int(location, value, int_value),
 
-            Constant::Float { location, value } => self.fold_constant_float(location, value),
+            Constant::Float {
+                location,
+                value,
+                float_value,
+            } => self.fold_constant_float(location, value, float_value),
 
             Constant::String { location, value } => self.fold_constant_string(location, value),
 
-            Constant::Tuple { location, elements } => self.fold_constant_tuple(location, elements),
+            Constant::Tuple {
+                location,
+                elements,
+                type_: (),
+            } => self.fold_constant_tuple(location, elements),
 
             Constant::List {
                 location,
@@ -960,6 +982,25 @@ pub trait UntypedConstantFolder {
                 record_constructor: _,
             } => self.fold_constant_record(location, module, name, arguments),
 
+            Constant::RecordUpdate {
+                location,
+                constructor_location,
+                module,
+                name,
+                record,
+                arguments,
+                tag: (),
+                type_: (),
+                field_map: _,
+            } => self.fold_constant_record_update(
+                location,
+                constructor_location,
+                module,
+                name,
+                record,
+                arguments,
+            ),
+
             Constant::BitArray { location, segments } => {
                 self.fold_constant_bit_array(location, segments)
             }
@@ -981,7 +1022,8 @@ pub trait UntypedConstantFolder {
             Constant::Invalid {
                 location,
                 type_: (),
-            } => self.fold_constant_invalid(location),
+                extra_information,
+            } => self.fold_constant_invalid(location, extra_information),
         }
     }
 
@@ -998,8 +1040,17 @@ pub trait UntypedConstantFolder {
         }
     }
 
-    fn fold_constant_float(&mut self, location: SrcSpan, value: EcoString) -> UntypedConstant {
-        Constant::Float { location, value }
+    fn fold_constant_float(
+        &mut self,
+        location: SrcSpan,
+        value: EcoString,
+        float_value: LiteralFloatValue,
+    ) -> UntypedConstant {
+        Constant::Float {
+            location,
+            value,
+            float_value,
+        }
     }
 
     fn fold_constant_string(&mut self, location: SrcSpan, value: EcoString) -> UntypedConstant {
@@ -1011,7 +1062,11 @@ pub trait UntypedConstantFolder {
         location: SrcSpan,
         elements: Vec<UntypedConstant>,
     ) -> UntypedConstant {
-        Constant::Tuple { location, elements }
+        Constant::Tuple {
+            location,
+            elements,
+            type_: (),
+        }
     }
 
     fn fold_constant_list(
@@ -1040,11 +1095,33 @@ pub trait UntypedConstantFolder {
             arguments,
             tag: (),
             type_: (),
-            field_map: None,
+            field_map: Inferred::Unknown,
             record_constructor: None,
         }
     }
 
+    fn fold_constant_record_update(
+        &mut self,
+        location: SrcSpan,
+        constructor_location: SrcSpan,
+        module: Option<(EcoString, SrcSpan)>,
+        name: EcoString,
+        record: RecordBeingUpdated<UntypedConstant>,
+        arguments: Vec<RecordUpdateArg<UntypedConstant>>,
+    ) -> UntypedConstant {
+        Constant::RecordUpdate {
+            location,
+            constructor_location,
+            module,
+            name,
+            record,
+            arguments,
+            tag: (),
+            type_: (),
+            field_map: Inferred::Unknown,
+        }
+    }
+
     fn fold_constant_bit_array(
         &mut self,
         location: SrcSpan,
@@ -1081,10 +1158,15 @@ pub trait UntypedConstantFolder {
         }
     }
 
-    fn fold_constant_invalid(&mut self, location: SrcSpan) -> UntypedConstant {
+    fn fold_constant_invalid(
+        &mut self,
+        location: SrcSpan,
+        extra_information: Option<InvalidExpression>,
+    ) -> UntypedConstant {
         Constant::Invalid {
             location,
             type_: (),
+            extra_information,
         }
     }
 
@@ -1143,6 +1225,42 @@ pub trait UntypedConstantFolder {
                 }
             }
 
+            Constant::RecordUpdate {
+                location,
+                constructor_location,
+                module,
+                name,
+                record,
+                arguments,
+                tag,
+                type_,
+                field_map,
+            } => {
+                let record = RecordBeingUpdated {
+                    base: Box::new(self.fold_constant(*record.base)),
+                    location: record.location,
+                };
+                let arguments = arguments
+                    .into_iter()
+                    .map(|argument| RecordUpdateArg {
+                        label: argument.label,
+                        location: argument.location,
+                        value: self.fold_constant(argument.value),
+                    })
+                    .collect();
+                Constant::RecordUpdate {
+                    location,
+                    constructor_location,
+                    module,
+                    name,
+                    record,
+                    arguments,
+                    tag,
+                    type_,
+                    field_map,
+                }
+            }
+
             Constant::BitArray { location, segments } => {
                 let segments = segments
                     .into_iter()
@@ -1188,7 +1306,11 @@ pub trait PatternFolder {
                 int_value,
             } => self.fold_pattern_int(location, value, int_value),
 
-            Pattern::Float { location, value } => self.fold_pattern_float(location, value),
+            Pattern::Float {
+                location,
+                value,
+                float_value,
+            } => self.fold_pattern_float(location, value, float_value),
 
             Pattern::String { location, value } => self.fold_pattern_string(location, value),
 
@@ -1277,8 +1399,17 @@ pub trait PatternFolder {
         }
     }
 
-    fn fold_pattern_float(&mut self, location: SrcSpan, value: EcoString) -> UntypedPattern {
-        Pattern::Float { location, value }
+    fn fold_pattern_float(
+        &mut self,
+        location: SrcSpan,
+        value: EcoString,
+        float_value: LiteralFloatValue,
+    ) -> UntypedPattern {
+        Pattern::Float {
+            location,
+            value,
+            float_value,
+        }
     }
 
     fn fold_pattern_string(&mut self, location: SrcSpan, value: EcoString) -> UntypedPattern {
@@ -1382,7 +1513,7 @@ pub trait PatternFolder {
         &mut self,
         location: SrcSpan,
         elements: Vec<UntypedPattern>,
-        tail: Option<Box<UntypedPattern>>,
+        tail: Option<Box<UntypedTailPattern>>,
     ) -> UntypedPattern {
         Pattern::List {
             location,
@@ -1490,7 +1621,12 @@ pub trait PatternFolder {
                     .into_iter()
                     .map(|pattern| self.fold_pattern(pattern))
                     .collect();
-                let tail = tail.map(|pattern| Box::new(self.fold_pattern(*pattern)));
+                let tail = tail.map(|tail_pattern| {
+                    Box::new(TailPattern {
+                        location: tail_pattern.location,
+                        pattern: self.fold_pattern(tail_pattern.pattern),
+                    })
+                });
                 Pattern::List {
                     location,
                     elements,
