@@ -354,7 +354,7 @@ impl<'a> CallGraphBuilder<'a> {
                     self.pattern(element);
                 }
                 if let Some(tail) = tail {
-                    self.pattern(tail);
+                    self.pattern(&tail.pattern);
                 }
             }
 
@@ -494,6 +494,16 @@ impl<'a> CallGraphBuilder<'a> {
                 }
             }
 
+            Constant::RecordUpdate {
+                record, arguments, ..
+            } => {
+                self.constant(&record.base);
+
+                for argument in arguments {
+                    self.constant(&argument.value);
+                }
+            }
+
             Constant::Var {
                 module: None, name, ..
             } => self.referenced(name),
