@@ -52,13 +52,29 @@ impl<'de> Deserialize<'de> for SpdxLicense {
     where
         D: serde::Deserializer<'de>,
     {
-        let s: &str = Deserialize::deserialize(deserializer)?;
-        match spdx::license_id(s) {
+        deserializer.deserialize_str(SpdxLicenseVisitor)
+    }
+}
+
+struct SpdxLicenseVisitor;
+
+impl<'de> serde::de::Visitor<'de> for SpdxLicenseVisitor {
+    type Value = SpdxLicense;
+
+    fn expecting(&self, formatter: &mut fmt::Formatter<'_>) -> fmt::Result {
+        formatter.write_str("a SPDX License ID")
+    }
+
+    fn visit_str<E>(self, value: &str) -> Result<Self::Value, E>
+    where
+        E: serde::de::Error,
+    {
+        match spdx::license_id(value) {
             None => Err(serde::de::Error::custom(format!(
-                "{s} is not a valid SPDX License ID"
+                "{value} is not a valid SPDX License ID"
             ))),
             Some(_) => Ok(SpdxLicense {
-                licence: String::from(s),
+                licence: value.to_string(),
             }),
         }
     }
@@ -218,13 +234,7 @@ impl PackageConfig {
         fs: &FS,
     ) -> Result<PackageConfig, Error> {
         let toml = fs.read(path.as_ref())?;
-        let config: PackageConfig = toml::from_str(&toml).map_err(|e| Error::FileIo {
-            action: FileIoAction::Parse,
-            kind: FileKind::File,
-            path: path.as_ref().to_path_buf(),
-            err: Some(e.to_string()),
-        })?;
-        Ok(config)
+        deserialise_config(path, toml)
     }
 
     /// Get the locked packages for the current config and a given (optional)
@@ -309,7 +319,8 @@ impl PackageConfig {
                 | Repository::Codeberg { tag_prefix, .. }
                 | Repository::SourceHut { tag_prefix, .. }
                 | Repository::Gitea { tag_prefix, .. }
-                | Repository::Forgejo { tag_prefix, .. },
+                | Repository::Forgejo { tag_prefix, .. }
+                | Repository::Tangled { tag_prefix, .. },
             ) => tag_prefix.as_ref(),
 
             Some(Repository::Custom { .. }) | None => None,
@@ -322,6 +333,35 @@ impl PackageConfig {
     }
 }
 
+fn deserialise_config<P: AsRef<Utf8Path>>(
+    path: P,
+    toml: String,
+) -> std::result::Result<PackageConfig, Error> {
+    let config: PackageConfig = toml::from_str(&toml).map_err(|e| Error::FileIo {
+        action: FileIoAction::Parse,
+        kind: FileKind::File,
+        path: path.as_ref().to_path_buf(),
+        err: Some(e.to_string()),
+    })?;
+    Ok(config)
+}
+
+// https://github.com/gleam-lang/gleam/issues/4867
+#[test]
+fn deny_extra_deps_properties() {
+    let toml = r#"
+name = "wibble"
+version = "1.0.0"
+
+[dependencies]
+aide_generator = { git = "git@github.com:crowdhailer/aide.git", ref = "f559c5bc", extra = "idk what this is" }
+"#;
+    let error = deserialise_config("gleam.toml", toml.into())
+        .expect_err("should fail to deserialise because of additional path");
+
+    insta::assert_snapshot!(insta::internals::AutoName, error.pretty_string());
+}
+
 #[test]
 fn locked_no_manifest() {
     let mut config = PackageConfig::default();
@@ -668,8 +708,16 @@ impl Default for PackageConfig {
 
 #[derive(Deserialize, Serialize, Debug, PartialEq, Eq, Default, Clone)]
 pub struct ErlangConfig {
+    /// An module that can be set in the `.app` file as the entrypoint for a stateful application
+    /// that defines a singleton supervision tree.
+    /// Erlang syntax.
     #[serde(default)]
     pub application_start_module: Option<EcoString>,
+    /// The argument for the start module start function. If not set then `[]` is used as the
+    /// default argument.
+    /// Erlang syntax.
+    #[serde(default)]
+    pub application_start_argument: Option<EcoString>,
     #[serde(default)]
     pub extra_applications: Vec<EcoString>,
 }
@@ -852,6 +900,14 @@ pub enum Repository {
         #[serde(rename = "tag-prefix")]
         tag_prefix: Option<String>,
     },
+    #[serde(rename = "tangled")]
+    Tangled {
+        user: String,
+        repo: String,
+        path: Option<String>,
+        #[serde(rename = "tag-prefix")]
+        tag_prefix: Option<String>,
+    },
     #[serde(rename = "custom")]
     Custom {
         url: String,
@@ -878,6 +934,9 @@ impl Repository {
             Repository::SourceHut { repo, user, .. } => {
                 format!("https://git.sr.ht/~{user}/{repo}")
             }
+            Repository::Tangled { repo, user, .. } => {
+                format!("https://tangled.sh/{user}/{repo}")
+            }
             Repository::Gitea {
                 repo, user, host, ..
             }
@@ -899,6 +958,7 @@ impl Repository {
             | Repository::BitBucket { path, .. }
             | Repository::Codeberg { path, .. }
             | Repository::SourceHut { path, .. }
+            | Repository::Tangled { path, .. }
             | Repository::Gitea { path, .. }
             | Repository::Forgejo { path, .. } => path.as_ref(),
 
@@ -1007,7 +1067,7 @@ mod package_name {
     use ecow::EcoString;
     use regex::Regex;
     use serde::Deserializer;
-    use std::sync::OnceLock;
+    use std::{fmt, sync::OnceLock};
 
     static PACKAGE_NAME_PATTERN: OnceLock<Regex> = OnceLock::new();
 
@@ -1015,16 +1075,32 @@ mod package_name {
     where
         D: Deserializer<'de>,
     {
-        let name: &str = serde::de::Deserialize::deserialize(deserializer)?;
-        if PACKAGE_NAME_PATTERN
-            .get_or_init(|| Regex::new("^[a-z][a-z0-9_]*$").expect("Package name regex"))
-            .is_match(name)
+        deserializer.deserialize_str(NameVisitor)
+    }
+
+    struct NameVisitor;
+
+    impl<'de> serde::de::Visitor<'de> for NameVisitor {
+        type Value = EcoString;
+
+        fn expecting(&self, formatter: &mut fmt::Formatter<'_>) -> fmt::Result {
+            formatter.write_str("a package name")
+        }
+
+        fn visit_str<E>(self, value: &str) -> Result<Self::Value, E>
+        where
+            E: serde::de::Error,
         {
-            Ok(name.into())
-        } else {
-            let error =
-                "Package names may only contain lowercase letters, numbers, and underscores";
-            Err(serde::de::Error::custom(error))
+            if PACKAGE_NAME_PATTERN
+                .get_or_init(|| Regex::new("^[a-z][a-z0-9_]*$").expect("Package name regex"))
+                .is_match(value)
+            {
+                Ok(value.into())
+            } else {
+                let error =
+                    "Package names may only contain lowercase letters, numbers, and underscores";
+                Err(serde::de::Error::custom(error))
+            }
         }
     }
 }
@@ -1034,12 +1110,13 @@ fn name_with_dash() {
     let input = r#"
 name = "one-two"
 "#;
-    assert_eq!(
+
+    insta::assert_snapshot!(
+        insta::internals::AutoName,
         toml::from_str::<PackageConfig>(input)
             .unwrap_err()
-            .to_string(),
-        "Package names may only contain lowercase letters, numbers, and underscores for key `name` at line 1 column 1"
-    )
+            .to_string()
+    );
 }
 
 #[test]
@@ -1047,11 +1124,11 @@ fn name_with_number_start() {
     let input = r#"
 name = "1"
 "#;
-    assert_eq!(
+    insta::assert_snapshot!(
+        insta::internals::AutoName,
         toml::from_str::<PackageConfig>(input)
             .unwrap_err()
             .to_string(),
-        "Package names may only contain lowercase letters, numbers, and underscores for key `name` at line 1 column 1"
     )
 }
 
