@@ -438,3 +438,57 @@ pub fn main(x) {
 "#,
     );
 }
+
+#[test]
+fn assert_const_concat() {
+    assert_erl!(
+        r#"
+const cute = "cute"
+const cute_bee = cute <> "bee"
+
+pub fn main() {
+  cute_bee
+}
+"#
+    );
+}
+
+#[test]
+fn assert_const_concat_many_strings() {
+    assert_erl!(
+        r#"
+const big_concat = "a" <> "b" <> "c" <> "d" <> "e" <> "f" <> "g" <> "h" <> "i" <> "j" <> "k" <> "l" <> "m" <> "n" <> "o" <> "p" <> "q" <> "r" <> "s" <> "t" <> "u" <> "v" <> "w" <> "x" <> "y" <> "z"
+
+pub fn main() {
+  big_concat
+}
+"#
+    );
+}
+
+#[test]
+fn assert_const_concat_many_strings_in_list() {
+    assert_erl!(
+        r#"
+const big_concat_list = ["a" <> "b" <> "c" <> "d" <> "e" <> "f" <> "g" <> "h" <> "i" <> "j" <> "k" <> "l" <> "m" <> "n" <> "o" <> "p" <> "q" <> "r" <> "s" <> "t" <> "u" <> "v" <> "w" <> "x" <> "y" <> "z"]
+
+pub fn main() {
+  big_concat_list
+}
+"#
+    );
+}
+
+#[test]
+fn assert_const_concat_other_const_concat() {
+    assert_erl!(
+        r#"
+const cute_bee = "cute" <> "bee"
+const cute_cute_bee_buzz = cute_bee <> "buzz"
+
+pub fn main() {
+  cute_cute_bee_buzz
+}
+"#
+    );
+}
