@@ -6,6 +6,7 @@ mod bit_array;
 mod blocks;
 mod cases;
 mod conditional_compilation;
+mod constant;
 mod custom_type;
 mod echo;
 mod external_fn;
@@ -6733,3 +6734,46 @@ fn format_panic_as_with_block_message() {
 "#
     );
 }
+
+// https://github.com/gleam-lang/gleam/issues/5056
+#[test]
+fn remove_redundant_negation_from_literal_int_1() {
+    assert_format_rewrite!(
+        "pub fn main() {
+  --1
+}
+",
+        "pub fn main() {
+  1
+}
+"
+    );
+}
+
+#[test]
+fn remove_redundant_negation_from_literal_int_2() {
+    assert_format_rewrite!(
+        "pub fn main() {
+  ---1
+}
+",
+        "pub fn main() {
+  -1
+}
+"
+    );
+}
+
+#[test]
+fn remove_redundant_negation_from_literal_int_3() {
+    assert_format_rewrite!(
+        "pub fn main() {
+  ----1
+}
+",
+        "pub fn main() {
+  1
+}
+"
+    );
+}
