@@ -124,9 +124,9 @@ use crate::{
     analyse::Inferred,
     ast::{
         self, ArgNames, Assert, AssignName, Assignment, AssignmentKind, BitArrayOption,
-        BitArraySegment, BitArraySize, CallArg, Clause, Definition, FunctionLiteralKind, Pattern,
-        PipelineAssignmentKind, Publicity, SrcSpan, Statement, TypedArg, TypedAssert,
-        TypedAssignment, TypedBitArraySize, TypedClause, TypedDefinition, TypedExpr,
+        BitArraySegment, BitArraySize, CallArg, Clause, FunctionLiteralKind, Pattern,
+        PipelineAssignmentKind, Publicity, SrcSpan, Statement, TailPattern, TypedArg, TypedAssert,
+        TypedAssignment, TypedBitArraySize, TypedClause, TypedDefinitions, TypedExpr,
         TypedExprBitArraySegment, TypedFunction, TypedModule, TypedPattern,
         TypedPipelineAssignment, TypedStatement, TypedUse, visit::Visit,
     },
@@ -148,11 +148,16 @@ pub fn module(
 ) -> TypedModule {
     let mut inliner = Inliner::new(modules);
 
-    module.definitions = module
-        .definitions
-        .into_iter()
-        .map(|definition| inliner.definition(definition))
-        .collect();
+    module.definitions = TypedDefinitions {
+        functions: module
+            .definitions
+            .functions
+            .into_iter()
+            .map(|function| inliner.function(function))
+            .collect(),
+        ..module.definitions
+    };
+
     module
 }
 
@@ -244,19 +249,6 @@ impl Inliner<'_> {
         self.renamed_variables.get(&name).cloned().unwrap_or(name)
     }
 
-    /// Perform inlining over a single definition. This only does anything for
-    /// function definitions as none of the other definitions can contain call
-    /// expressions to be inlined.
-    fn definition(&mut self, definition: TypedDefinition) -> TypedDefinition {
-        match definition {
-            Definition::Function(function_ast) => Definition::Function(self.function(function_ast)),
-            Definition::TypeAlias(_)
-            | Definition::CustomType(_)
-            | Definition::Import(_)
-            | Definition::ModuleConstant(_) => definition,
-        }
-    }
-
     fn function(&mut self, mut function: TypedFunction) -> TypedFunction {
         for argument in function.arguments.iter() {
             match &argument.names {
@@ -369,7 +361,12 @@ impl Inliner<'_> {
                     .into_iter()
                     .map(|element| self.register_pattern_variables(element))
                     .collect(),
-                tail: tail.map(|tail| Box::new(self.register_pattern_variables(*tail))),
+                tail: tail.map(|tail| {
+                    Box::new(TailPattern {
+                        location: tail.location,
+                        pattern: self.register_pattern_variables(tail.pattern),
+                    })
+                }),
                 type_,
             },
             Pattern::Constructor {
@@ -540,7 +537,6 @@ impl Inliner<'_> {
                     }
                 }
                 ValueConstructorVariant::ModuleConstant { .. }
-                | ValueConstructorVariant::LocalConstant { .. }
                 | ValueConstructorVariant::ModuleFn { .. }
                 | ValueConstructorVariant::Record { .. } => expression,
             },
@@ -632,6 +628,18 @@ impl Inliner<'_> {
                 documentation,
             },
 
+            TypedExpr::PositionalAccess {
+                location,
+                type_,
+                index,
+                record,
+            } => TypedExpr::PositionalAccess {
+                location,
+                type_,
+                index,
+                record: self.boxed_expression(record),
+            },
+
             TypedExpr::Tuple {
                 location,
                 type_,
@@ -852,7 +860,6 @@ impl Inliner<'_> {
                 // function calls, so they also cannot be inlined.
                 ValueConstructorVariant::LocalVariable { .. }
                 | ValueConstructorVariant::ModuleConstant { .. }
-                | ValueConstructorVariant::LocalConstant { .. }
                 | ValueConstructorVariant::Record { .. } => function,
             },
             TypedExpr::ModuleSelect {
@@ -907,6 +914,7 @@ impl Inliner<'_> {
             | TypedExpr::BinOp { .. }
             | TypedExpr::Case { .. }
             | TypedExpr::RecordAccess { .. }
+            | TypedExpr::PositionalAccess { .. }
             | TypedExpr::Tuple { .. }
             | TypedExpr::TupleIndex { .. }
             | TypedExpr::Todo { .. }
@@ -1577,7 +1585,6 @@ impl FunctionToInlinable {
                         }
                     }
                     ValueConstructorVariant::ModuleConstant { .. }
-                    | ValueConstructorVariant::LocalConstant { .. }
                     | ValueConstructorVariant::ModuleFn { .. }
                     | ValueConstructorVariant::Record { .. } => {}
                 }
@@ -1621,6 +1628,7 @@ impl FunctionToInlinable {
             | TypedExpr::List { .. }
             | TypedExpr::BinOp { .. }
             | TypedExpr::RecordAccess { .. }
+            | TypedExpr::PositionalAccess { .. }
             | TypedExpr::ModuleSelect { .. }
             | TypedExpr::Tuple { .. }
             | TypedExpr::TupleIndex { .. }
@@ -1680,8 +1688,7 @@ impl FunctionToInlinable {
             ValueConstructorVariant::LocalVariable { .. } => {
                 Some(InlinableValueConstructor::LocalVariable)
             }
-            ValueConstructorVariant::ModuleConstant { .. }
-            | ValueConstructorVariant::LocalConstant { .. } => None,
+            ValueConstructorVariant::ModuleConstant { .. } => None,
             ValueConstructorVariant::ModuleFn { name, module, .. } => {
                 Some(InlinableValueConstructor::Function {
                     name: name.clone(),
