@@ -2,9 +2,8 @@ use crate::{
     Error, Result, Warning,
     analyse::name::correct_name_case,
     ast::{
-        self, Constant, CustomType, Definition, DefinitionLocation, ModuleConstant,
-        PatternUnusedArguments, SrcSpan, TypedArg, TypedConstant, TypedExpr, TypedFunction,
-        TypedModule, TypedPattern,
+        self, Constant, CustomType, DefinitionLocation, ModuleConstant, PatternUnusedArguments,
+        SrcSpan, TypedArg, TypedConstant, TypedExpr, TypedFunction, TypedModule, TypedPattern,
     },
     build::{
         ExpressionPosition, Located, Module, UnqualifiedImport, type_constructor_from_modules,
@@ -13,13 +12,14 @@ use crate::{
     io::{BeamCompiler, CommandExecutor, FileSystemReader, FileSystemWriter},
     language_server::{
         code_action::{
-            AddOmittedLabels, CollapseNestedCase, ExtractFunction, RemoveBlock,
+            AddOmittedLabels, CollapseNestedCase, ExtractFunction, MergeCaseBranches, RemoveBlock,
             RemovePrivateOpaque, RemoveUnreachableCaseClauses,
         },
         compiler::LspProjectCompiler,
         files::FileSystemProxy,
         progress::ProgressReporter,
         reference::FindVariableReferences,
+        rename::RenameOutcome,
     },
     line_numbers::LineNumbers,
     paths::ProjectPaths,
@@ -34,6 +34,7 @@ use camino::Utf8PathBuf;
 use ecow::EcoString;
 use itertools::Itertools;
 use lsp::CodeAction;
+use lsp_server::ResponseError;
 use lsp_types::{
     self as lsp, DocumentSymbol, Hover, HoverContents, MarkedString, Position,
     PrepareRenameResponse, Range, SignatureHelp, SymbolKind, SymbolTag, TextEdit, Url,
@@ -44,9 +45,9 @@ use std::{collections::HashSet, sync::Arc};
 use super::{
     DownloadDependencies, MakeLocker,
     code_action::{
-        AddAnnotations, CodeActionBuilder, ConvertFromUse, ConvertToFunctionCall, ConvertToPipe,
-        ConvertToUse, ExpandFunctionCapture, ExtractConstant, ExtractVariable,
-        FillInMissingLabelledArgs, FillUnusedFields, FixBinaryOperation,
+        AddAnnotations, AnnotateTopLevelDefinitions, CodeActionBuilder, ConvertFromUse,
+        ConvertToFunctionCall, ConvertToPipe, ConvertToUse, ExpandFunctionCapture, ExtractConstant,
+        ExtractVariable, FillInMissingLabelledArgs, FillUnusedFields, FixBinaryOperation,
         FixTruncatedBitArraySegment, GenerateDynamicDecoder, GenerateFunction, GenerateJsonEncoder,
         GenerateVariant, InlineVariable, InterpolateString, LetAssertToCase, PatternMatchOnValue,
         RedundantTupleInCaseSubject, RemoveEchos, RemoveUnusedImports, UseLabelShorthandSyntax,
@@ -270,7 +271,7 @@ where
                 None => return Ok(None),
             };
 
-            let completer = Completer::new(&src, &params, &this.compiler, module);
+            let mut completer = Completer::new(&src, &params, &this.compiler, module);
             let byte_index = completer.module_line_numbers.byte_index(params.position);
 
             // If in comment context, do not provide completions
@@ -278,7 +279,7 @@ where
                 return Ok(None);
             }
 
-            // Check current filercontents if the user is writing an import
+            // Check current file contents if the user is writing an import
             // and handle separately from the rest of the completion flow
             // Check if an import is being written
             if let Some(value) = completer.import_completions() {
@@ -308,9 +309,10 @@ where
                     Some(completions)
                 }
                 Located::Expression {
-                    expression: TypedExpr::RecordAccess { record, .. },
+                    expression: TypedExpr::RecordAccess { record, type_, .. },
                     ..
                 } => {
+                    completer.expected_type = Some(type_.clone());
                     let mut completions = vec![];
                     completions.append(&mut completer.completion_values());
                     completions.append(&mut completer.completion_field_accessors(record.type_()));
@@ -331,28 +333,30 @@ where
                     );
                     Some(completions)
                 }
-                Located::Statement(_) | Located::Expression { .. } => {
+                Located::Expression { expression, .. } => {
+                    completer.expected_type = Some(expression.type_());
                     Some(completer.completion_values())
                 }
-                Located::ModuleStatement(Definition::Function(_)) => {
-                    Some(completer.completion_types())
-                }
+                Located::ModuleFunction(_) => Some(completer.completion_types()),
+
+                Located::Statement(_) => Some(completer.completion_values()),
 
                 Located::FunctionBody(_) => Some(completer.completion_values()),
 
-                Located::ModuleStatement(Definition::TypeAlias(_) | Definition::CustomType(_))
+                Located::ModuleTypeAlias(_)
+                | Located::ModuleCustomType(_)
                 | Located::VariantConstructorDefinition(_) => Some(completer.completion_types()),
 
                 // If the import completions returned no results and we are in an import then
                 // we should try to provide completions for unqualified values
-                Located::ModuleStatement(Definition::Import(import)) => this
+                Located::ModuleImport(import) => this
                     .compiler
                     .get_module_interface(import.module.as_str())
                     .map(|importing_module| {
                         completer.unqualified_completions_from_module(importing_module, true)
                     }),
 
-                Located::ModuleStatement(Definition::ModuleConstant(_)) | Located::Constant(_) => {
+                Located::ModuleConstant(_) | Located::Constant(_) => {
                     Some(completer.completion_values())
                 }
 
@@ -418,6 +422,7 @@ where
                 &this.error,
                 &mut actions,
             );
+            actions.extend(MergeCaseBranches::new(module, &lines, &params).code_actions());
             actions.extend(FixBinaryOperation::new(module, &lines, &params).code_actions());
             actions
                 .extend(FixTruncatedBitArraySegment::new(module, &lines, &params).code_actions());
@@ -462,6 +467,8 @@ where
             )
             .code_actions();
             AddAnnotations::new(module, &lines, &params).code_action(&mut actions);
+            actions
+                .extend(AnnotateTopLevelDefinitions::new(module, &lines, &params).code_actions());
             Ok(if actions.is_empty() {
                 None
             } else {
@@ -481,134 +488,121 @@ where
             };
             let line_numbers = LineNumbers::new(&module.code);
 
-            for definition in &module.ast.definitions {
-                match definition {
-                    // Typically, imports aren't considered document symbols.
-                    Definition::Import(_) => {}
+            for function in &module.ast.definitions.functions {
+                // By default, the function's location ends right after the return type.
+                // For the full symbol range, have it end at the end of the body.
+                // Also include the documentation, if available.
+                //
+                // By convention, the symbol span starts from the leading slash in the
+                // documentation comment's marker ('///'), not from its content (of which
+                // we have the position), so we must convert the content start position
+                // to the leading slash's position using 'get_doc_marker_pos'.
+                let full_function_span = SrcSpan {
+                    start: function
+                        .documentation
+                        .as_ref()
+                        .map(|(doc_start, _)| get_doc_marker_pos(*doc_start))
+                        .unwrap_or(function.location.start),
 
-                    Definition::Function(function) => {
-                        // By default, the function's location ends right after the return type.
-                        // For the full symbol range, have it end at the end of the body.
-                        // Also include the documentation, if available.
-                        //
-                        // By convention, the symbol span starts from the leading slash in the
-                        // documentation comment's marker ('///'), not from its content (of which
-                        // we have the position), so we must convert the content start position
-                        // to the leading slash's position using 'get_doc_marker_pos'.
-                        let full_function_span = SrcSpan {
-                            start: function
-                                .documentation
-                                .as_ref()
-                                .map(|(doc_start, _)| get_doc_marker_pos(*doc_start))
-                                .unwrap_or(function.location.start),
+                    end: function.end_position,
+                };
 
-                            end: function.end_position,
-                        };
+                let (name_location, name) = function
+                    .name
+                    .as_ref()
+                    .expect("Function in a definition must be named");
 
-                        let (name_location, name) = function
-                            .name
-                            .as_ref()
-                            .expect("Function in a definition must be named");
+                // The 'deprecated' field is deprecated, but we have to specify it anyway
+                // to be able to construct the 'DocumentSymbol' type, so
+                // we suppress the warning. We specify 'None' as specifying 'Some'
+                // is what is actually deprecated.
+                #[allow(deprecated)]
+                symbols.push(DocumentSymbol {
+                    name: name.to_string(),
+                    detail: Some(
+                        Printer::new(&module.ast.names)
+                            .print_type(&get_function_type(function))
+                            .to_string(),
+                    ),
+                    kind: SymbolKind::FUNCTION,
+                    tags: make_deprecated_symbol_tag(&function.deprecation),
+                    deprecated: None,
+                    range: src_span_to_lsp_range(full_function_span, &line_numbers),
+                    selection_range: src_span_to_lsp_range(*name_location, &line_numbers),
+                    children: None,
+                });
+            }
 
-                        // The 'deprecated' field is deprecated, but we have to specify it anyway
-                        // to be able to construct the 'DocumentSymbol' type, so
-                        // we suppress the warning. We specify 'None' as specifying 'Some'
-                        // is what is actually deprecated.
-                        #[allow(deprecated)]
-                        symbols.push(DocumentSymbol {
-                            name: name.to_string(),
-                            detail: Some(
-                                Printer::new(&module.ast.names)
-                                    .print_type(&get_function_type(function))
-                                    .to_string(),
-                            ),
-                            kind: SymbolKind::FUNCTION,
-                            tags: make_deprecated_symbol_tag(&function.deprecation),
-                            deprecated: None,
-                            range: src_span_to_lsp_range(full_function_span, &line_numbers),
-                            selection_range: src_span_to_lsp_range(*name_location, &line_numbers),
-                            children: None,
-                        });
+            for alias in &module.ast.definitions.type_aliases {
+                let full_alias_span = match alias.documentation {
+                    Some((doc_position, _)) => {
+                        SrcSpan::new(get_doc_marker_pos(doc_position), alias.location.end)
                     }
+                    None => alias.location,
+                };
 
-                    Definition::TypeAlias(alias) => {
-                        let full_alias_span = match alias.documentation {
-                            Some((doc_position, _)) => {
-                                SrcSpan::new(get_doc_marker_pos(doc_position), alias.location.end)
-                            }
-                            None => alias.location,
-                        };
+                // The 'deprecated' field is deprecated, but we have to specify it anyway
+                // to be able to construct the 'DocumentSymbol' type, so
+                // we suppress the warning. We specify 'None' as specifying 'Some'
+                // is what is actually deprecated.
+                #[allow(deprecated)]
+                symbols.push(DocumentSymbol {
+                    name: alias.alias.to_string(),
+                    detail: Some(
+                        Printer::new(&module.ast.names)
+                            // If we print with aliases, we end up printing the alias which the user
+                            // is currently hovering, which is not helpful. Instead, we print the
+                            // raw type, so the user can see which type the alias represents
+                            .print_type_without_aliases(&alias.type_)
+                            .to_string(),
+                    ),
+                    kind: SymbolKind::CLASS,
+                    tags: make_deprecated_symbol_tag(&alias.deprecation),
+                    deprecated: None,
+                    range: src_span_to_lsp_range(full_alias_span, &line_numbers),
+                    selection_range: src_span_to_lsp_range(alias.name_location, &line_numbers),
+                    children: None,
+                });
+            }
 
-                        // The 'deprecated' field is deprecated, but we have to specify it anyway
-                        // to be able to construct the 'DocumentSymbol' type, so
-                        // we suppress the warning. We specify 'None' as specifying 'Some'
-                        // is what is actually deprecated.
-                        #[allow(deprecated)]
-                        symbols.push(DocumentSymbol {
-                            name: alias.alias.to_string(),
-                            detail: Some(
-                                Printer::new(&module.ast.names)
-                                    // If we print with aliases, we end up printing the alias which the user
-                                    // is currently hovering, which is not helpful. Instead, we print the
-                                    // raw type, so the user can see which type the alias represents
-                                    .print_type_without_aliases(&alias.type_)
-                                    .to_string(),
-                            ),
-                            kind: SymbolKind::CLASS,
-                            tags: make_deprecated_symbol_tag(&alias.deprecation),
-                            deprecated: None,
-                            range: src_span_to_lsp_range(full_alias_span, &line_numbers),
-                            selection_range: src_span_to_lsp_range(
-                                alias.name_location,
-                                &line_numbers,
-                            ),
-                            children: None,
-                        });
-                    }
+            for custom_type in &module.ast.definitions.custom_types {
+                symbols.push(custom_type_symbol(custom_type, &line_numbers, module));
+            }
 
-                    Definition::CustomType(type_) => {
-                        symbols.push(custom_type_symbol(type_, &line_numbers, module));
-                    }
+            for constant in &module.ast.definitions.constants {
+                // `ModuleConstant.location` ends at the constant's name or type.
+                // For the full symbol span, necessary for `range`, we need to
+                // include the constant value as well.
+                // Also include the documentation at the start, if available.
+                let full_constant_span = SrcSpan {
+                    start: constant
+                        .documentation
+                        .as_ref()
+                        .map(|(doc_start, _)| get_doc_marker_pos(*doc_start))
+                        .unwrap_or(constant.location.start),
 
-                    Definition::ModuleConstant(constant) => {
-                        // `ModuleConstant.location` ends at the constant's name or type.
-                        // For the full symbol span, necessary for `range`, we need to
-                        // include the constant value as well.
-                        // Also include the documentation at the start, if available.
-                        let full_constant_span = SrcSpan {
-                            start: constant
-                                .documentation
-                                .as_ref()
-                                .map(|(doc_start, _)| get_doc_marker_pos(*doc_start))
-                                .unwrap_or(constant.location.start),
+                    end: constant.value.location().end,
+                };
 
-                            end: constant.value.location().end,
-                        };
-
-                        // The 'deprecated' field is deprecated, but we have to specify it anyway
-                        // to be able to construct the 'DocumentSymbol' type, so
-                        // we suppress the warning. We specify 'None' as specifying 'Some'
-                        // is what is actually deprecated.
-                        #[allow(deprecated)]
-                        symbols.push(DocumentSymbol {
-                            name: constant.name.to_string(),
-                            detail: Some(
-                                Printer::new(&module.ast.names)
-                                    .print_type(&constant.type_)
-                                    .to_string(),
-                            ),
-                            kind: SymbolKind::CONSTANT,
-                            tags: make_deprecated_symbol_tag(&constant.deprecation),
-                            deprecated: None,
-                            range: src_span_to_lsp_range(full_constant_span, &line_numbers),
-                            selection_range: src_span_to_lsp_range(
-                                constant.name_location,
-                                &line_numbers,
-                            ),
-                            children: None,
-                        });
-                    }
-                }
+                // The 'deprecated' field is deprecated, but we have to specify it anyway
+                // to be able to construct the 'DocumentSymbol' type, so
+                // we suppress the warning. We specify 'None' as specifying 'Some'
+                // is what is actually deprecated.
+                #[allow(deprecated)]
+                symbols.push(DocumentSymbol {
+                    name: constant.name.to_string(),
+                    detail: Some(
+                        Printer::new(&module.ast.names)
+                            .print_type(&constant.type_)
+                            .to_string(),
+                    ),
+                    kind: SymbolKind::CONSTANT,
+                    tags: make_deprecated_symbol_tag(&constant.deprecation),
+                    deprecated: None,
+                    range: src_span_to_lsp_range(full_constant_span, &line_numbers),
+                    selection_range: src_span_to_lsp_range(constant.name_location, &line_numbers),
+                    children: None,
+                });
             }
 
             Ok(symbols)
@@ -698,17 +692,20 @@ where
         })
     }
 
-    pub fn rename(&mut self, params: lsp::RenameParams) -> Response<Option<WorkspaceEdit>> {
+    pub fn rename(
+        &mut self,
+        params: lsp::RenameParams,
+    ) -> Response<Result<Option<WorkspaceEdit>, ResponseError>> {
         self.respond(|this| {
             let position = &params.text_document_position;
 
             let (lines, found) = match this.node_at_position(position) {
                 Some(value) => value,
-                None => return Ok(None),
+                None => return Ok(RenameOutcome::NoRenames.into_result()),
             };
 
             let Some(module) = this.module_for_uri(&position.text_document.uri) else {
-                return Ok(None);
+                return Ok(RenameOutcome::NoRenames.into_result());
             };
 
             Ok(match reference_for_ast_node(found, &module.name) {
@@ -719,7 +716,9 @@ where
                     ..
                 }) => {
                     let rename_kind = match origin.map(|origin| origin.syntax) {
-                        Some(VariableSyntax::Generated) => return Ok(None),
+                        Some(VariableSyntax::Generated) => {
+                            return Ok(RenameOutcome::NoRenames.into_result());
+                        }
                         Some(VariableSyntax::LabelShorthand(_)) => {
                             VariableReferenceKind::LabelShorthand
                         }
@@ -736,6 +735,7 @@ where
                         name,
                         rename_kind,
                     )
+                    .into_result()
                 }
                 Some(Referenced::ModuleValue {
                     module: module_name,
@@ -755,7 +755,9 @@ where
                         target_kind,
                         layer: ast::Layer::Value,
                     },
-                ),
+                )
+                .into_result(),
+
                 Some(Referenced::ModuleType {
                     module: module_name,
                     target_kind,
@@ -773,8 +775,10 @@ where
                         target_kind,
                         layer: ast::Layer::Type,
                     },
-                ),
-                None => None,
+                )
+                .into_result(),
+
+                None => RenameOutcome::NoRenames.into_result(),
             })
         })
     }
@@ -896,14 +900,14 @@ where
 
             Ok(match found {
                 Located::Statement(_) => None, // TODO: hover for statement
-                Located::ModuleStatement(Definition::Function(fun)) => {
-                    Some(hover_for_function_head(fun, lines, module))
+                Located::ModuleFunction(function) => {
+                    Some(hover_for_function_head(function, lines, module))
                 }
-                Located::ModuleStatement(Definition::ModuleConstant(constant)) => {
+                Located::ModuleConstant(constant) => {
                     Some(hover_for_module_constant(constant, lines, module))
                 }
                 Located::Constant(constant) => Some(hover_for_constant(constant, lines, module)),
-                Located::ModuleStatement(Definition::Import(import)) => {
+                Located::ModuleImport(import) => {
                     let Some(module) = this.compiler.get_module_interface(&import.module) else {
                         return Ok(None);
                     };
@@ -914,7 +918,8 @@ where
                         &this.hex_deps,
                     ))
                 }
-                Located::ModuleStatement(_) => None,
+                Located::ModuleCustomType(_) => None,
+                Located::ModuleTypeAlias(_) => None,
                 Located::VariantConstructorDefinition(_) => None,
                 Located::UnqualifiedImport(UnqualifiedImport {
                     name,
@@ -1464,7 +1469,7 @@ fn code_action_unused_values(
     }
 
     // Sort spans by start position, with longer spans coming first
-    unused_values.sort_by_key(|span| (span.start, -(span.end as i64 - span.start as i64)));
+    unused_values.sort_by_key(|span| (span.start, -(span.len() as i64)));
 
     let mut processed_lsp_range = Vec::new();
 
@@ -1601,17 +1606,13 @@ fn get_hexdocs_link_section(
     ast: &TypedModule,
     hex_deps: &HashSet<EcoString>,
 ) -> Option<String> {
-    let package_name = ast
-        .definitions
-        .iter()
-        .find_map(|definition| match definition {
-            Definition::Import(import)
-                if import.module == module_name && hex_deps.contains(&import.package) =>
-            {
-                Some(&import.package)
-            }
-            _ => None,
-        })?;
+    let package_name = ast.definitions.imports.iter().find_map(|import| {
+        if import.module == module_name && hex_deps.contains(&import.package) {
+            Some(&import.package)
+        } else {
+            None
+        }
+    })?;
 
     Some(format_hexdocs_link_section(
         package_name,
