@@ -4,7 +4,7 @@ use super::{
 };
 use crate::{
     Result,
-    diagnostic::{Diagnostic, Level},
+    diagnostic::{Diagnostic, ExtraLabel, Level},
     io::{BeamCompiler, CommandExecutor, FileSystemReader, FileSystemWriter},
     language_server::{
         DownloadDependencies, MakeLocker,
@@ -18,7 +18,7 @@ use crate::{
 };
 use camino::{Utf8Path, Utf8PathBuf};
 use debug_ignore::DebugIgnore;
-use itertools::Itertools;
+use lsp_server::ResponseError;
 use lsp_types::{
     self as lsp, HoverProviderCapability, InitializeParams, Position, PublishDiagnosticsParams,
     Range, RenameOptions, TextEdit, Url,
@@ -98,7 +98,7 @@ where
     }
 
     fn handle_request(&mut self, id: lsp_server::RequestId, request: Request) {
-        let (payload, feedback) = match request {
+        let (outcome, feedback) = match request {
             Request::Format(param) => self.format(param),
             Request::Hover(param) => self.hover(param),
             Request::GoToDefinition(param) => self.goto_definition(param),
@@ -114,11 +114,19 @@ where
 
         self.publish_feedback(feedback);
 
-        let response = lsp_server::Response {
-            id,
-            error: None,
-            result: Some(payload),
+        let response = match outcome {
+            Ok(payload) => lsp_server::Response {
+                id,
+                error: None,
+                result: Some(payload),
+            },
+            Err(error) => lsp_server::Response {
+                id,
+                error: Some(error),
+                result: None,
+            },
         };
+
         self.connection
             .sender
             .send(lsp_server::Message::Response(response))
@@ -236,12 +244,33 @@ where
         &mut self,
         path: Utf8PathBuf,
         handler: Handler,
-    ) -> (Json, Feedback)
+    ) -> (Result<Json, ResponseError>, Feedback)
     where
         T: serde::Serialize,
         Handler: FnOnce(
             &mut LanguageServerEngine<IO, ConnectionProgressReporter<'a>>,
         ) -> engine::Response<T>,
+    {
+        self.fallible_respond_with_engine(path, |engine| {
+            let response = handler(engine);
+            engine::Response {
+                result: response.result.map(Ok),
+                warnings: response.warnings,
+                compilation: response.compilation,
+            }
+        })
+    }
+
+    fn fallible_respond_with_engine<T, Handler>(
+        &mut self,
+        path: Utf8PathBuf,
+        handler: Handler,
+    ) -> (Result<Json, ResponseError>, Feedback)
+    where
+        T: serde::Serialize,
+        Handler: FnOnce(
+            &mut LanguageServerEngine<IO, ConnectionProgressReporter<'a>>,
+        ) -> engine::Response<Result<T, ResponseError>>,
     {
         match self.router.project_for_path(path) {
             Ok(Some(project)) => {
@@ -251,33 +280,47 @@ where
                     compilation,
                 } = handler(&mut project.engine);
                 match result {
-                    Ok(value) => {
+                    Ok(Ok(value)) => {
                         let feedback = project.feedback.response(compilation, warnings);
                         let json = serde_json::to_value(value).expect("response to json");
-                        (json, feedback)
+                        (Ok(json), feedback)
+                    }
+                    Ok(Err(error)) => {
+                        let feedback = project.feedback.response(compilation, warnings);
+                        (Err(error), feedback)
                     }
                     Err(e) => {
                         let feedback = project.feedback.build_with_error(e, compilation, warnings);
-                        (Json::Null, feedback)
+                        (Ok(Json::Null), feedback)
                     }
                 }
             }
 
-            Ok(None) => (Json::Null, Feedback::default()),
+            Ok(None) => (Ok(Json::Null), Feedback::default()),
 
-            Err(error) => (Json::Null, self.outside_of_project_feedback.error(error)),
+            Err(error) => (
+                Ok(Json::Null),
+                self.outside_of_project_feedback.error(error),
+            ),
         }
     }
 
-    fn path_error_response(&mut self, path: Utf8PathBuf, error: crate::Error) -> (Json, Feedback) {
+    fn path_error_response(
+        &mut self,
+        path: Utf8PathBuf,
+        error: crate::Error,
+    ) -> (Result<Json, ResponseError>, Feedback) {
         let feedback = match self.router.project_for_path(path) {
             Ok(Some(project)) => project.feedback.error(error),
             Ok(None) | Err(_) => self.outside_of_project_feedback.error(error),
         };
-        (Json::Null, feedback)
+        (Ok(Json::Null), feedback)
     }
 
-    fn format(&mut self, params: lsp::DocumentFormattingParams) -> (Json, Feedback) {
+    fn format(
+        &mut self,
+        params: lsp::DocumentFormattingParams,
+    ) -> (Result<Json, ResponseError>, Feedback) {
         let path = super::path(&params.text_document.uri);
         let mut new_text = String::new();
 
@@ -298,15 +341,18 @@ where
         };
         let json = serde_json::to_value(vec![edit]).expect("to JSON value");
 
-        (json, Feedback::default())
+        (Ok(json), Feedback::default())
     }
 
-    fn hover(&mut self, params: lsp::HoverParams) -> (Json, Feedback) {
+    fn hover(&mut self, params: lsp::HoverParams) -> (Result<Json, ResponseError>, Feedback) {
         let path = super::path(&params.text_document_position_params.text_document.uri);
         self.respond_with_engine(path, |engine| engine.hover(params))
     }
 
-    fn goto_definition(&mut self, params: lsp::GotoDefinitionParams) -> (Json, Feedback) {
+    fn goto_definition(
+        &mut self,
+        params: lsp::GotoDefinitionParams,
+    ) -> (Result<Json, ResponseError>, Feedback) {
         let path = super::path(&params.text_document_position_params.text_document.uri);
         self.respond_with_engine(path, |engine| engine.goto_definition(params))
     }
@@ -314,12 +360,15 @@ where
     fn goto_type_definition(
         &mut self,
         params: lsp_types::GotoDefinitionParams,
-    ) -> (Json, Feedback) {
+    ) -> (Result<Json, ResponseError>, Feedback) {
         let path = super::path(&params.text_document_position_params.text_document.uri);
         self.respond_with_engine(path, |engine| engine.goto_type_definition(params))
     }
 
-    fn completion(&mut self, params: lsp::CompletionParams) -> (Json, Feedback) {
+    fn completion(
+        &mut self,
+        params: lsp::CompletionParams,
+    ) -> (Result<Json, ResponseError>, Feedback) {
         let path = super::path(&params.text_document_position.text_document.uri);
 
         let src = match self.io.read(&path) {
@@ -331,32 +380,52 @@ where
         })
     }
 
-    fn signature_help(&mut self, params: lsp_types::SignatureHelpParams) -> (Json, Feedback) {
+    fn signature_help(
+        &mut self,
+        params: lsp_types::SignatureHelpParams,
+    ) -> (Result<Json, ResponseError>, Feedback) {
         let path = super::path(&params.text_document_position_params.text_document.uri);
         self.respond_with_engine(path, |engine| engine.signature_help(params))
     }
 
-    fn code_action(&mut self, params: lsp::CodeActionParams) -> (Json, Feedback) {
+    fn code_action(
+        &mut self,
+        params: lsp::CodeActionParams,
+    ) -> (Result<Json, ResponseError>, Feedback) {
         let path = super::path(&params.text_document.uri);
         self.respond_with_engine(path, |engine| engine.code_actions(params))
     }
 
-    fn document_symbol(&mut self, params: lsp::DocumentSymbolParams) -> (Json, Feedback) {
+    fn document_symbol(
+        &mut self,
+        params: lsp::DocumentSymbolParams,
+    ) -> (Result<Json, ResponseError>, Feedback) {
         let path = super::path(&params.text_document.uri);
         self.respond_with_engine(path, |engine| engine.document_symbol(params))
     }
 
-    fn prepare_rename(&mut self, params: lsp::TextDocumentPositionParams) -> (Json, Feedback) {
+    fn prepare_rename(
+        &mut self,
+        params: lsp::TextDocumentPositionParams,
+    ) -> (Result<Json, ResponseError>, Feedback) {
         let path = super::path(&params.text_document.uri);
         self.respond_with_engine(path, |engine| engine.prepare_rename(params))
     }
 
-    fn rename(&mut self, params: lsp::RenameParams) -> (Json, Feedback) {
+    fn rename(&mut self, params: lsp::RenameParams) -> (Result<Json, ResponseError>, Feedback) {
         let path = super::path(&params.text_document_position.text_document.uri);
-        self.respond_with_engine(path, |engine| engine.rename(params))
+        self.fallible_respond_with_engine(
+            path,
+            |engine: &mut LanguageServerEngine<IO, ConnectionProgressReporter<'a>>| {
+                engine.rename(params)
+            },
+        )
     }
 
-    fn find_references(&mut self, params: lsp_types::ReferenceParams) -> (Json, Feedback) {
+    fn find_references(
+        &mut self,
+        params: lsp_types::ReferenceParams,
+    ) -> (Result<Json, ResponseError>, Feedback) {
         let path = super::path(&params.text_document_position.text_document.uri);
         self.respond_with_engine(path, |engine| engine.find_references(params))
     }
@@ -511,28 +580,6 @@ fn diagnostic_to_lsp(diagnostic: Diagnostic) -> Vec<lsp::Diagnostic> {
     let path = path_to_uri(location.path);
     let range = src_span_to_lsp_range(location.label.span, &line_numbers);
 
-    let related_info = location
-        .extra_labels
-        .iter()
-        .map(|extra| {
-            let message = extra.label.text.clone().unwrap_or_default();
-            let location = match &extra.src_info {
-                Some((src, path)) => {
-                    let line_numbers = LineNumbers::new(src);
-                    lsp::Location {
-                        uri: path_to_uri(path.clone()),
-                        range: src_span_to_lsp_range(extra.label.span, &line_numbers),
-                    }
-                }
-                _ => lsp::Location {
-                    uri: path.clone(),
-                    range: src_span_to_lsp_range(extra.label.span, &line_numbers),
-                },
-            };
-            lsp::DiagnosticRelatedInformation { location, message }
-        })
-        .collect_vec();
-
     let main = lsp::Diagnostic {
         range,
         severity: Some(severity),
@@ -540,11 +587,13 @@ fn diagnostic_to_lsp(diagnostic: Diagnostic) -> Vec<lsp::Diagnostic> {
         code_description: None,
         source: None,
         message: text,
-        related_information: if related_info.is_empty() {
-            None
-        } else {
-            Some(related_info)
-        },
+        related_information: related_information(
+            &hint,
+            &location.extra_labels,
+            &path,
+            &line_numbers,
+            range,
+        ),
         tags: None,
         data: None,
     };
@@ -554,6 +603,13 @@ fn diagnostic_to_lsp(diagnostic: Diagnostic) -> Vec<lsp::Diagnostic> {
             let hint = lsp::Diagnostic {
                 severity: Some(lsp::DiagnosticSeverity::HINT),
                 message: hint,
+                // Some editors require this kind of "link" to group diagnostics.
+                // For example, in Zed "go to next diagnostic" would move you from
+                // the warning to the hint in the same location without this.
+                related_information: Some(vec![lsp::DiagnosticRelatedInformation {
+                    location: lsp::Location { uri: path, range },
+                    message: String::new(),
+                }]),
                 ..main.clone()
             };
             vec![main, hint]
@@ -562,6 +618,54 @@ fn diagnostic_to_lsp(diagnostic: Diagnostic) -> Vec<lsp::Diagnostic> {
     }
 }
 
+fn related_information(
+    hint: &Option<String>,
+    extra_labels: &[ExtraLabel],
+    path: &Url,
+    line_numbers: &LineNumbers,
+    range: Range,
+) -> Option<Vec<lsp::DiagnosticRelatedInformation>> {
+    let mut related_info = Vec::with_capacity(extra_labels.len() + 1);
+
+    // The hint is included as a dedicated diagnostic _and_ the related information
+    // to maximize compatibility
+    if let Some(hint) = hint {
+        let hint = lsp::DiagnosticRelatedInformation {
+            message: hint.clone(),
+            location: lsp::Location {
+                uri: path.clone(),
+                range,
+            },
+        };
+        related_info.push(hint);
+    }
+
+    let additional_info = extra_labels.iter().map(|extra| {
+        let message = extra.label.text.clone().unwrap_or_default();
+        let location = match &extra.src_info {
+            Some((src, path)) => {
+                let line_numbers = LineNumbers::new(src);
+                lsp::Location {
+                    uri: path_to_uri(path.clone()),
+                    range: src_span_to_lsp_range(extra.label.span, &line_numbers),
+                }
+            }
+            _ => lsp::Location {
+                uri: path.clone(),
+                range: src_span_to_lsp_range(extra.label.span, line_numbers),
+            },
+        };
+        lsp::DiagnosticRelatedInformation { location, message }
+    });
+    related_info.extend(additional_info);
+
+    if related_info.is_empty() {
+        None
+    } else {
+        Some(related_info)
+    }
+}
+
 fn path_to_uri(path: Utf8PathBuf) -> Url {
     let mut file: String = "file://".into();
     file.push_str(&path.as_os_str().to_string_lossy());
