@@ -405,13 +405,13 @@ fn importable_adds_extra_new_line_if_import_exists_below_other_definitions() {
 #[test]
 fn importable_does_not_add_extra_new_line_if_imports_exist() {
     let dep = "pub fn wobble() {\nNil\n}";
-    let prefix = "import foo\n\n";
+    let prefix = "import wibble\n\n";
     let code = "";
 
     assert_completion_with_prefix!(
         TestProject::for_source(code)
             .add_module("dep", dep)
-            .add_module("foo", ""),
+            .add_module("wibble", ""),
         prefix
     );
 }
@@ -425,7 +425,7 @@ fn importable_does_not_add_extra_new_line_if_newline_exists() {
     assert_completion_with_prefix!(
         TestProject::for_source(code)
             .add_module("dep", dep)
-            .add_module("foo", ""),
+            .add_module("wibble", ""),
         prefix
     );
 }
@@ -972,11 +972,11 @@ pub fn main() {
 #[test]
 fn local_variable_inside_nested_exprs() {
     let code = r#"
-type Foo { Bar(List(#(Bool))) }
+type Wibble { Wobble(List(#(Bool))) }
 fn wibble() {
-  Bar([#(!{
-    let foo = True
-    foo
+  Wobble([#(!{
+    let wibble = True
+    wibble
   })])
   todo
 }
@@ -2252,3 +2252,45 @@ const x = "io."
     );
     assert_eq!(completions, vec![],);
 }
+
+#[test]
+fn prefer_values_matching_expected_type() {
+    let code = "
+pub fn main() -> Bool {
+  let wibble = 123
+  let wubble = True
+  let Wobble = 1.5
+  w
+}
+";
+
+    assert_completion!(TestProject::for_source(code), Position::new(5, 3));
+}
+
+#[test]
+fn prefer_function_which_returns_expected_type() {
+    let code = "
+pub fn main() -> Int {
+  a
+}
+
+fn add(a, b) { a + b }
+fn sub(a, b) { a - b }
+fn addf(a, b) { a +. b }
+";
+
+    assert_completion!(TestProject::for_source(code), Position::new(2, 3));
+}
+
+#[test]
+fn prefer_function_which_returns_expected_generic_type() {
+    let code = "
+pub fn main() -> Result(Int, Nil) {
+  let result = Ok(12)
+  let result2 = Error(True)
+  r
+}
+";
+
+    assert_completion!(TestProject::for_source(code), Position::new(4, 3));
+}
