@@ -6,12 +6,14 @@ use itertools::Itertools;
 
 use crate::{
     Result,
+    analyse::Inferred,
     ast::{
         BitArrayOption, BitArraySegment, CallArg, Constant, Publicity, SrcSpan, TypedConstant,
         TypedConstantBitArraySegment, TypedConstantBitArraySegmentOption,
     },
     build::Origin,
     line_numbers::{Character, LineNumbers},
+    parse::LiteralFloatValue,
     reference::{Reference, ReferenceKind, ReferenceMap},
     schema_capnp::{self as schema, *},
     type_::{
@@ -375,7 +377,7 @@ impl ModuleDecoder {
             Which::Int(reader) => Ok(self.constant_int(self.str(reader?)?)),
             Which::Float(reader) => Ok(self.constant_float(self.str(reader?)?)),
             Which::String(reader) => Ok(self.constant_string(self.str(reader?)?)),
-            Which::Tuple(reader) => self.constant_tuple(&reader?),
+            Which::Tuple(reader) => self.constant_tuple(&reader),
             Which::List(reader) => self.constant_list(&reader),
             Which::Record(reader) => self.constant_record(&reader),
             Which::BitArray(reader) => self.constant_bit_array(&reader?),
@@ -396,6 +398,8 @@ impl ModuleDecoder {
         Constant::Float {
             location: Default::default(),
             value: value.into(),
+            float_value: LiteralFloatValue::parse(value)
+                .expect("float value to parse as non-NaN f64"),
         }
     }
 
@@ -406,13 +410,12 @@ impl ModuleDecoder {
         }
     }
 
-    fn constant_tuple(
-        &mut self,
-        reader: &capnp::struct_list::Reader<'_, constant::Owned>,
-    ) -> Result<TypedConstant> {
+    fn constant_tuple(&mut self, reader: &constant::tuple::Reader<'_>) -> Result<TypedConstant> {
+        let type_ = self.type_(&reader.get_type()?)?;
         Ok(Constant::Tuple {
             location: Default::default(),
-            elements: read_vec!(reader, self, constant),
+            elements: read_vec!(reader.get_elements()?, self, constant),
+            type_,
         })
     }
 
@@ -436,7 +439,7 @@ impl ModuleDecoder {
             arguments,
             tag,
             type_,
-            field_map: None,
+            field_map: Inferred::Unknown,
             record_constructor: None,
         })
     }
@@ -683,6 +686,11 @@ impl ModuleDecoder {
                 self,
                 variant_specific_accessors
             ),
+            variant_positional_accessors: read_vec!(
+                &reader.get_positional_accessors()?,
+                self,
+                positional_accessors
+            ),
         })
     }
 
@@ -697,6 +705,13 @@ impl ModuleDecoder {
         ))
     }
 
+    fn positional_accessors(
+        &mut self,
+        reader: &positional_accessors::Reader<'_>,
+    ) -> Result<Vec<Arc<Type>>> {
+        Ok(read_vec!(&reader.get_accessors()?, self, type_))
+    }
+
     fn record_accessor(&mut self, reader: &record_accessor::Reader<'_>) -> Result<RecordAccessor> {
         Ok(RecordAccessor {
             index: reader.get_index() as u64,
