@@ -4,12 +4,14 @@ use type_::{AccessorsMap, FieldMap, RecordAccessor};
 
 use super::*;
 use crate::{
+    analyse::Inferred,
     ast::{
         BitArrayOption, BitArraySegment, CallArg, Constant, Publicity, SrcSpan, TypedConstant,
         TypedConstantBitArraySegmentOption,
     },
     build::Origin,
     line_numbers::LineNumbers,
+    parse::LiteralFloatValue,
     reference::{Reference, ReferenceKind},
     type_::{
         self, Deprecation, ModuleInterface, Opaque, References, Type, TypeAliasConstructor,
@@ -937,6 +939,7 @@ fn accessors() {
                     type_: type_::int(),
                     shared_accessors: accessors1.clone().into(),
                     variant_specific_accessors: vec![accessors1.into()],
+                    variant_positional_accessors: vec![vec![type_::int(), type_::float()]],
                 },
             ),
             (
@@ -946,6 +949,7 @@ fn accessors() {
                     type_: type_::int(),
                     shared_accessors: accessors2.clone().into(),
                     variant_specific_accessors: vec![accessors2.into()],
+                    variant_positional_accessors: vec![vec![]],
                 },
             ),
         ]
@@ -1015,6 +1019,7 @@ fn private_accessors() {
                     type_: type_::int(),
                     shared_accessors: accessors1.clone().into(),
                     variant_specific_accessors: vec![accessors1.into()],
+                    variant_positional_accessors: vec![vec![type_::int(), type_::float()]],
                 },
             ),
             (
@@ -1024,6 +1029,7 @@ fn private_accessors() {
                     type_: type_::int(),
                     shared_accessors: accessors2.clone().into(),
                     variant_specific_accessors: vec![accessors2.into()],
+                    variant_positional_accessors: vec![vec![]],
                 },
             ),
         ]
@@ -1056,6 +1062,7 @@ fn constant_float() {
     let module = constant_module(Constant::Float {
         location: Default::default(),
         value: "1.0".into(),
+        float_value: LiteralFloatValue::ONE,
     });
 
     assert_eq!(roundtrip(&module), module);
@@ -1073,8 +1080,14 @@ fn constant_string() {
 
 #[test]
 fn constant_tuple() {
+    let int_float_tuple_type = type_::tuple(vec![type_::int(), type_::float()]);
     let module = constant_module(Constant::Tuple {
         location: Default::default(),
+        type_: type_::tuple(vec![
+            type_::int(),
+            type_::float(),
+            int_float_tuple_type.clone(),
+        ]),
         elements: vec![
             Constant::Int {
                 location: Default::default(),
@@ -1084,9 +1097,11 @@ fn constant_tuple() {
             Constant::Float {
                 location: Default::default(),
                 value: "1.0".into(),
+                float_value: LiteralFloatValue::ONE,
             },
             Constant::Tuple {
                 location: Default::default(),
+                type_: int_float_tuple_type,
                 elements: vec![
                     Constant::Int {
                         location: Default::default(),
@@ -1096,6 +1111,7 @@ fn constant_tuple() {
                     Constant::Float {
                         location: Default::default(),
                         value: "1.0".into(),
+                        float_value: LiteralFloatValue::ONE,
                     },
                 ],
             },
@@ -1146,6 +1162,7 @@ fn constant_record() {
                 value: Constant::Float {
                     location: Default::default(),
                     value: "0.0".into(),
+                    float_value: LiteralFloatValue::ZERO,
                 },
             },
             CallArg {
@@ -1161,7 +1178,7 @@ fn constant_record() {
         ],
         tag: "thetag".into(),
         type_: type_::int(),
-        field_map: None,
+        field_map: Inferred::Unknown,
         record_constructor: None,
     });
 
