@@ -363,18 +363,6 @@ fn pointless_spread() {
     );
 }
 
-// https://github.com/gleam-lang/gleam/issues/1358
-#[test]
-fn lowcase_bool_in_pattern() {
-    assert_error!(
-        "case 42 > 42 { true -> 1; false -> 2; }",
-        ParseError {
-            error: ParseErrorType::LowcaseBooleanPattern,
-            location: SrcSpan { start: 15, end: 19 },
-        }
-    );
-}
-
 // https://github.com/gleam-lang/gleam/issues/1613
 #[test]
 fn anonymous_function_labeled_arguments() {
@@ -817,12 +805,23 @@ fn attributes_with_no_definition() {
     );
 }
 
+#[test]
+fn external_attribute_with_custom_type() {
+    assert_parse_module!(
+        r#"
+@external(erlang, "gleam_stdlib", "dict")
+@external(javascript, "./gleam_stdlib.d.ts", "Dict")
+pub type Dict(key, value)
+"#
+    );
+}
+
 #[test]
 fn external_attribute_with_non_fn_definition() {
     assert_module_error!(
         r#"
 @external(erlang, "module", "fun")
-pub type Fun
+pub type Fun = Fun
 "#
     );
 }
@@ -1145,6 +1144,34 @@ fn main() {
     );
 }
 
+#[test]
+fn case_clause_no_subject() {
+    assert_module_error!(
+        "
+fn main() {
+    case 1 {
+      -> 1
+      _ -> 2
+    }
+}
+"
+    );
+}
+
+#[test]
+fn case_alternative_clause_no_subject() {
+    assert_module_error!(
+        "
+fn main() {
+    case 1 {
+      1 | -> 1
+      _ -> 1
+    }
+}
+"
+    );
+}
+
 #[test]
 fn use_invalid_assignments() {
     assert_module_error!(
@@ -1992,3 +2019,63 @@ pub fn main() {
 "#
     );
 }
+
+#[test]
+fn wrong_function_return_type_declaration_using_colon_instead_of_right_arrow() {
+    assert_module_error!(
+        r#"
+pub fn main(): Nil {}
+        "#
+    );
+}
+
+#[test]
+fn const_record_update_basic() {
+    assert_parse_module!(
+        r#"
+type Person {
+  Person(name: String, age: Int)
+}
+
+const alice = Person("Alice", 30)
+const bob = Person(..alice, name: "Bob")
+"#
+    );
+}
+
+#[test]
+fn const_record_update_all_fields() {
+    assert_parse_module!(
+        r#"
+type Person {
+  Person(name: String, age: Int, city: String)
+}
+
+const base = Person("Alice", 30, "London")
+const updated = Person(..base, name: "Bob", age: 25, city: "Paris")
+"#
+    );
+}
+
+#[test]
+fn const_record_update_only() {
+    assert_parse_module!(
+        r#"
+type Person {
+  Person(name: String, age: Int)
+}
+
+const alice = Person("Alice", 30)
+const bob = Person(..alice)
+"#
+    );
+}
+
+#[test]
+fn const_record_update_with_module() {
+    assert_parse_module!(
+        r#"
+const local_const = other.Record(..other.base, field: value)
+"#
+    );
+}
