@@ -131,6 +131,13 @@ impl Type {
         }
     }
 
+    pub fn is_named(&self) -> bool {
+        match self {
+            Self::Named { .. } => true,
+            _ => false,
+        }
+    }
+
     pub fn result_ok_type(&self) -> Option<Arc<Type>> {
         match self {
             Self::Named {
@@ -558,7 +565,11 @@ impl Type {
                     && package == other_package
                     && module == other_module
                     && name == other_name
-                    && arguments == other_arguments
+                    && arguments.len() == other_arguments.len()
+                    && arguments
+                        .iter()
+                        .zip(other_arguments)
+                        .all(|(one, other)| one.same_as(other))
             }
 
             (Type::Fn { .. }, Type::Named { .. } | Type::Tuple { .. }) => false,
@@ -653,6 +664,7 @@ pub struct AccessorsMap {
     pub type_: Arc<Type>,
     pub shared_accessors: HashMap<EcoString, RecordAccessor>,
     pub variant_specific_accessors: Vec<HashMap<EcoString, RecordAccessor>>,
+    pub variant_positional_accessors: Vec<Vec<Arc<Type>>>,
 }
 
 impl AccessorsMap {
@@ -664,6 +676,11 @@ impl AccessorsMap {
             .and_then(|index| self.variant_specific_accessors.get(index as usize))
             .unwrap_or(&self.shared_accessors)
     }
+
+    pub fn positional_accessors(&self, inferred_variant: u16) -> Option<&Vec<Arc<Type>>> {
+        self.variant_positional_accessors
+            .get(inferred_variant as usize)
+    }
 }
 
 #[derive(Debug, Clone, PartialEq, Eq)]
@@ -693,11 +710,6 @@ pub enum ValueConstructorVariant {
         implementations: Implementations,
     },
 
-    /// A constant defined locally, for example when pattern matching on string literals
-    LocalConstant {
-        literal: Constant<Arc<Type>, EcoString>,
-    },
-
     /// A function belonging to the module
     ModuleFn {
         name: EcoString,
@@ -763,12 +775,6 @@ impl ValueConstructorVariant {
                 documentation: documentation.clone(),
             },
 
-            Self::LocalConstant { literal } => ModuleValueConstructor::Constant {
-                literal: literal.clone(),
-                location: literal.location(),
-                documentation: None,
-            },
-
             Self::LocalVariable { location, .. } => ModuleValueConstructor::Fn {
                 name: function_name.clone(),
                 module: module_name.clone(),
@@ -809,7 +815,6 @@ impl ValueConstructorVariant {
             | ValueConstructorVariant::ModuleConstant { location, .. }
             | ValueConstructorVariant::ModuleFn { location, .. }
             | ValueConstructorVariant::Record { location, .. } => *location,
-            ValueConstructorVariant::LocalConstant { literal } => literal.location(),
         }
     }
 
@@ -826,7 +831,6 @@ impl ValueConstructorVariant {
                 matches!(origin.syntax, VariableSyntax::Generated)
             }
             ValueConstructorVariant::ModuleConstant { .. }
-            | ValueConstructorVariant::LocalConstant { .. }
             | ValueConstructorVariant::ModuleFn { .. }
             | ValueConstructorVariant::Record { .. } => false,
         }
@@ -850,7 +854,6 @@ impl ValueConstructorVariant {
     pub fn implementations(&self) -> Implementations {
         match self {
             ValueConstructorVariant::Record { .. }
-            | ValueConstructorVariant::LocalConstant { .. }
             | ValueConstructorVariant::LocalVariable { .. } => Implementations {
                 gleam: true,
                 can_run_on_erlang: true,
@@ -872,7 +875,6 @@ impl ValueConstructorVariant {
         match self {
             ValueConstructorVariant::LocalVariable { .. }
             | ValueConstructorVariant::ModuleConstant { .. }
-            | ValueConstructorVariant::LocalConstant { .. }
             | ValueConstructorVariant::ModuleFn { .. } => None,
             ValueConstructorVariant::Record { field_map, .. } => field_map.as_ref(),
         }
@@ -1203,7 +1205,8 @@ impl TypeVar {
     pub fn is_unbound(&self) -> bool {
         match self {
             Self::Unbound { .. } => true,
-            Self::Link { .. } | Self::Generic { .. } => false,
+            Self::Link { type_ } => type_.is_unbound(),
+            Self::Generic { .. } => false,
         }
     }
 
@@ -1375,10 +1378,13 @@ pub struct ValueConstructor {
     pub type_: Arc<Type>,
 }
 
-#[derive(Debug, Clone, PartialEq, Eq, Serialize)]
+#[derive(Debug, Clone, PartialEq, Eq, Serialize, Default)]
 pub enum Deprecation {
+    #[default]
     NotDeprecated,
-    Deprecated { message: EcoString },
+    Deprecated {
+        message: EcoString,
+    },
 }
 
 impl Deprecation {
@@ -1391,12 +1397,6 @@ impl Deprecation {
     }
 }
 
-impl Default for Deprecation {
-    fn default() -> Self {
-        Self::NotDeprecated
-    }
-}
-
 impl ValueConstructor {
     pub fn local_variable(location: SrcSpan, origin: VariableOrigin, type_: Arc<Type>) -> Self {
         Self {
@@ -1426,11 +1426,6 @@ impl ValueConstructor {
                 span: *location,
             },
 
-            ValueConstructorVariant::LocalConstant { literal } => DefinitionLocation {
-                module: None,
-                span: literal.location(),
-            },
-
             ValueConstructorVariant::LocalVariable { location, .. } => DefinitionLocation {
                 module: None,
                 span: *location,
@@ -1440,8 +1435,7 @@ impl ValueConstructor {
 
     pub(crate) fn get_documentation(&self) -> Option<&str> {
         match &self.variant {
-            ValueConstructorVariant::LocalConstant { .. }
-            | ValueConstructorVariant::LocalVariable { .. } => Some("A locally defined variable."),
+            ValueConstructorVariant::LocalVariable { .. } => Some("A locally defined variable."),
 
             ValueConstructorVariant::ModuleFn { documentation, .. }
             | ValueConstructorVariant::Record { documentation, .. }
@@ -1475,8 +1469,7 @@ impl ValueConstructor {
             // `Purity::Unknown`. See the documentation for the `Purity` type
             // for more information on why this is the case.
             ValueConstructorVariant::LocalVariable { .. }
-            | ValueConstructorVariant::ModuleConstant { .. }
-            | ValueConstructorVariant::LocalConstant { .. } => Purity::Unknown,
+            | ValueConstructorVariant::ModuleConstant { .. } => Purity::Unknown,
 
             // Constructing records is always pure
             ValueConstructorVariant::Record { .. } => Purity::Pure,
@@ -1528,46 +1521,41 @@ fn assert_no_labelled_arguments<A>(arguments: &[CallArg<A>]) -> Result<(), Error
 /// could cause naively-implemented type checking to diverge.
 /// While traversing the type tree.
 ///
-fn unify_unbound_type(type_: Arc<Type>, own_id: u64) -> Result<(), UnifyError> {
-    if let Type::Var { type_ } = type_.deref() {
-        let new_value = match type_.borrow().deref() {
-            TypeVar::Link { type_, .. } => return unify_unbound_type(type_.clone(), own_id),
+fn unify_unbound_type(type_: &Type, own_id: u64) -> Result<(), UnifyError> {
+    if let Type::Var { type_ } = type_ {
+        return match type_.borrow().deref() {
+            TypeVar::Link { type_, .. } => unify_unbound_type(type_, own_id),
 
             TypeVar::Unbound { id } => {
                 if id == &own_id {
-                    return Err(UnifyError::RecursiveType);
+                    Err(UnifyError::RecursiveType)
                 } else {
-                    Some(TypeVar::Unbound { id: *id })
+                    Ok(())
                 }
             }
 
-            TypeVar::Generic { .. } => return Ok(()),
+            TypeVar::Generic { .. } => Ok(()),
         };
-
-        if let Some(t) = new_value {
-            *type_.borrow_mut() = t;
-        }
-        return Ok(());
     }
 
-    match type_.deref() {
+    match type_ {
         Type::Named { arguments, .. } => {
             for argument in arguments {
-                unify_unbound_type(argument.clone(), own_id)?
+                unify_unbound_type(argument, own_id)?
             }
             Ok(())
         }
 
         Type::Fn { arguments, return_ } => {
             for argument in arguments {
-                unify_unbound_type(argument.clone(), own_id)?;
+                unify_unbound_type(argument, own_id)?;
             }
-            unify_unbound_type(return_.clone(), own_id)
+            unify_unbound_type(return_, own_id)
         }
 
         Type::Tuple { elements, .. } => {
             for element in elements {
-                unify_unbound_type(element.clone(), own_id)?
+                unify_unbound_type(element, own_id)?
             }
             Ok(())
         }
