@@ -385,6 +385,11 @@ fn wrong_number_of_subjects() {
     assert_error!("case 1 { _, _ -> 1 }");
 }
 
+#[test]
+fn wrong_number_of_subjects_alternative_patterns() {
+    assert_error!("case 1 { _, _ | _ | _, _, _ -> 1 }");
+}
+
 #[test]
 fn recursive_var() {
     assert_error!("let id = fn(x) { x(x) } 1");
@@ -3345,3 +3350,52 @@ fn type_used_as_a_constructor_with_more_arguments() {
 }"
     );
 }
+
+#[test]
+fn remembering_record_field_when_type_checking_fails() {
+    assert_module_error!(
+        r#"pub type Wibble {
+  Wibble(x: Int, f: fn(Wobble) -> Int)
+}
+
+pub fn wibble() {
+  Wibble(1, fn(_) { 2 })
+}
+
+pub fn wobble(wibble: Wibble) {
+  wibble.f
+}
+
+pub fn woo(wibble: Wibble) {
+  Wibble(..wibble, x: 1)
+}"#
+    );
+}
+
+#[test]
+fn external_annotation_on_custom_type_with_constructors() {
+    assert_module_error!(
+        r#"
+@external(erlang, "gleam_stdlib", "dict")
+pub type Dict(key, value) {
+  Dict(pairs: List(#(key, value)))
+}
+"#
+    );
+}
+
+#[test]
+fn generic_unlabelled_field_in_updated_record_wrong_type() {
+    assert_module_error!(
+        "
+pub type Wibble(a) {
+  Wibble(a, b: Int, c: a)
+}
+
+pub fn main() {
+  let w = Wibble(1, 2, 3)
+  Wibble(..w, c: False)
+}
+"
+    );
+}
