@@ -169,6 +169,17 @@ pub enum Warning {
     DeprecatedEnvironmentVariable {
         variable: DeprecatedEnvironmentVariable,
     },
+
+    EmptyModule {
+        path: Utf8PathBuf,
+        name: EcoString,
+    },
+
+    DetachedDocComment {
+        path: Utf8PathBuf,
+        src: EcoString,
+        location: SrcSpan,
+    },
 }
 
 #[derive(Debug, Clone, Eq, PartialEq, Copy)]
@@ -419,6 +430,29 @@ To have a clause without a guard, remove this.",
                 }
             }
 
+            Warning::DetachedDocComment {
+                path,
+                src,
+                location,
+            } => Diagnostic {
+                title: "Detached doc comment".into(),
+                text: wrap(
+                    "This doc comment is followed by a regular \
+comment so it is not attached to any definition.",
+                ),
+                level: diagnostic::Level::Warning,
+                location: Some(Location {
+                    path: path.to_path_buf(),
+                    src: src.clone(),
+                    label: diagnostic::Label {
+                        text: Some("This is not attached to a definition".into()),
+                        span: *location,
+                    },
+                    extra_labels: Vec::new(),
+                }),
+                hint: Some("Move the comment above the doc comment".into()),
+            },
+
             Warning::Type { path, warning, src } => match warning {
                 type_::Warning::Todo {
                     kind,
@@ -1251,6 +1285,9 @@ See: https://tour.gleam.run/functions/pipelines/",
                             "Use of unaligned bit arrays on the JavaScript target was"
                         }
                         FeatureKind::BoolAssert => "The bool `assert` statement was",
+                        FeatureKind::ExternalCustomType => {
+                            "The `@external` annotation on custom types was"
+                        }
                     };
 
                     Diagnostic {
@@ -1488,6 +1525,14 @@ The imported value could not be used in this module anyway."
                     location: None,
                 }
             }
+
+            Warning::EmptyModule { path: _, name } => Diagnostic {
+                title: "Empty module".into(),
+                text: format!("Module '{name}' contains no public definitions."),
+                hint: Some("You can safely remove this module.".into()),
+                level: diagnostic::Level::Warning,
+                location: None,
+            },
         }
     }
 
