@@ -1,14 +1,14 @@
 -module(ffi_erlang).
 
 -export([
-    to_string/1, append/2, print/1, file_exists/1, halt/1, to_dynamic/1
+    to_string/1, append/2, print/1, file_exists/1, halt/1, to_dynamic/1, to_codepoint/1
 ]).
 
 append(A, B) ->
     <<A/binary, B/binary>>.
 
 print(S) ->
-    io:format("~s", [S]), 
+    io:format("~s", [S]),
     nil.
 
 to_string(Term) ->
@@ -23,3 +23,5 @@ halt(Code) ->
 
 to_dynamic(X) ->
     X.
+
+to_codepoint(X) -> X.
